/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data;

import java.io.Serializable;

import jp.littlesoft.data.Table;
import jp.littlesoft.data.exception.DataException;

/**
 * テーブルオブジェクトに行データを読み込むクラスを作成する為のインターフェース。<br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface DataProvider extends Serializable {

  public void setParams(Params<?>  param);
  
  
  public Params<?> getParams();

  /**
   * テーブルオブジェクトのカラムを構築するメソッド<br/>
   * 
   * @param  table テーブルオブジェクト
   * @throws DataException 例外発生時
   */
  public <E extends Serializable> void setupColumn(Table<?> table) throws DataException;
  
  /**
   * テーブルオブジェクトにデータをロードするメソッド<br/>
   * 
   * @param table テーブルオブジェクト
   * @throws DataException 例外発生時
   */
  public void loadData(Table<?> table) throws DataException;
  
  /**
   * テーブルオブジェクトに追加でロードできるデータ行が検査するメソッド<br/>
   * 
   * @param table テーブルオブジェクト
   * @param toLast True：最終行まで調べる、False：最終行まで調べない
   * @return True：追加行が存在する、False：存在しない
   * @throws DataException 例外発生時
   */
  public boolean checkMoreRow(Table<?> table, boolean toLast) throws DataException;
  
  /**
   * 内部リソースの開放を行います。
   * 
   * @throws DataException 例外発生時
   */
  public void free() throws DataException;

}
