/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data;

import java.io.Serializable;

import jp.littlesoft.data.event.ResolvingListener;
import jp.littlesoft.data.exception.DataException;

/**
 * テーブルオブジェクトの内容をデータベースあるいはデータファイル等に書き出すクラスを作成する為のインターフェース。<br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface DataResolver extends Serializable {

  /**
   * テーブルオブジェクトの内容をデータベースあるいはデータファイル等に書き出す処理を実装します。
   * 
   * @param dataTable Table オブジェクト
   * @throws DataException 例外発生時
   */
  public void execute(Table<?> dataTable) throws DataException;
  
  /**
   * ResolvingListenerを削除します。<br/>
   *
   * @param l ResolvingListenerインターフェースを実装したオブジェクト
   */
  public void removeResolvingListener(ResolvingListener l);
  
  /**
   * ResolvingListenerを登録します。<br/>
   *
   * @param l ResolvingListenerインターフェースを実装したオブジェクト
   */
  public void addResolvingListener(ResolvingListener l);

}
