/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.event;

import java.util.*;
import java.io.Serializable;
import jp.littlesoft.data.Column;

/**
 * データ変更イベント。<br/><br/>
 *
 * Tableクラス、Paramsクラスなどのデータの内容が変更された際のイベントオブジェクトです。<br/>
 * このイベントを受け取るには、受け取り側クラスでChangeListenerインターフェースを実装し、
 * 対象のデータオブジェクトaddChangeListenerで受け取り側オブジェクトを登録します。<br/><br/>
 *
 * データ項目の値が変更された場合は、当クラスのoldValue､newValueプロパティに変更される前後のそれぞれの値が設定されます。<br/><br/>
 *
 * @see jp.littlesoft.data.event.ChangeListener
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class ChangeEvent extends EventObject implements Serializable {

  //変更されたカラム
  private Column column;
  //変更後の値
  private Object newValue;
  //変更前の値
  private Object oldValue;

  /**
   * コンストラクタ。<br/>
   * 
   * @param  source  イベントの発生元のデータオブジェクト
   */
  public ChangeEvent(Object source) {
    super(source);
  }
  
  /**
   * コンストラクタ。<br/>
   * 
   * @param  source        イベントの発生元のデータオブジェクト
   * @param  changeColumn  値が変更されたカラムオブジェクト
   * @param  argNewValue   変更後のカラム値
   * @param  argOldValue   変更前のカラム値
   */
  public ChangeEvent(Object source, Column changeColumn, Object argNewValue, Object argOldValue) {
    super(source);
    column = changeColumn;
    newValue = argNewValue;
    oldValue = argOldValue;
  }

  /**
   * 値が変更されたカラムオブジェクトを取得する。<br/>
   * 
   * @return  変更されるカラム
   */
  public jp.littlesoft.data.Column getColumn() {
    return column;
  }

  /**
   * 変更後の値を取得する。<br/>
   * 
   * @return  新しい値
   */
  public Object getNewValue() {
    return newValue;
  }

  /**
   * 変更されるカラムの現在の値または変更前の値。<br/>
   * 
   * @return  現在の値または変更前の値
   */
  public Object getOldValue() {
    return oldValue;
  }
}

