/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.event;

import java.io.Serializable;
import java.util.*;

import jp.littlesoft.data.Params;

/**
 * テーブルオブジェクトデータ永続化のイベントオブジェクト。<br/><br/>
 * 
 * このイベントを受け取るには、受け取り側クラスでResolvingListenerインターフェースを実装し、
 * 対象のデータオブジェクトaddResolvingListenerで受け取り側オブジェクトを登録します。<br/>
 *
 * @author LittleSoft Corporation
 * @version 1.3.0
 */ 
public class ResolvingEvent extends EventObject implements Serializable {

  private static final long serialVersionUID = 1L;

  /**
   * 永続化の種類を示す列挙形
   */
  public enum Type {DELETE, UPDATE, INSERT, ANY};

  private Type type;
  private jp.littlesoft.data.Row<?> oldRow;
  private jp.littlesoft.data.Row<?> newRow;
  private boolean abort;
  private String abortMessage;
  private String sql;
  private Params<?> params;

  /**
   * コンストラクタ。<br/>
   *
   * @param  source  Eventが最初に発生したオブジェクト
   */
  public ResolvingEvent(Object source) {
    super(source);
  }

  /**
   * イベントが発生したデータオブジェクトを設定する。<br/>
   *
   * @param newSource イベントが発生したオブジェクト
   */
  public void setSource(Object newSource) {
    this.source = newSource;
  }

  /**
   * SQLタイプを取得します。<br/><br/>
   *
   * 当クラスの定数が設定されています。<br/>
   *
   * @return SQLタイプ(DELETE, UPDATE, INSERT)
   */
  public Type getType() {
    return type;
  }

  /**
   * ＳＱＬタイプを設定します。<br/><br/>
   *
   * 当クラスの定数を設定します。<br/>
   *
   * @param type ＳＱＬタイプ
   */  
  public void setType(Type type) {
    this.type = type;
  }

  /**
   * OldRowを設定します。<br/>
   *
   * @param newOldRow OldRow
   */  
  public void setOldRow(jp.littlesoft.data.Row<?> newOldRow) {
    oldRow = newOldRow;
  }

  /**
   * OldRowを取得します。<br/>
   *
   * @return  OldRow
   */  
  public jp.littlesoft.data.Row<?> getOldRow() {
    return oldRow;
  }

  /**
   * NewRowを設定します。<br/>
   *
   * @param  newNewRow  NewRow
   */  
  public void setNewRow(jp.littlesoft.data.Row<?> newNewRow) {
    newRow = newNewRow;
  }

  /**
   * NewRowを取得します。<br/>
   *
   * @return  NewRow
   */
  public jp.littlesoft.data.Row<?> getNewRow() {
    return newRow;
  }


  /**
   * 処理を中断するかどうかを設定します。<br/><br/>
   *
   * ResolvingListener各メソッドの内で、引数として渡されるオブジェクトの当プロパティをtrueに設定すると処理を中断させることが可能です。<br/>
   * 
   * @param  newAbort  処理を中断するかどうか
   */
  public void setAbort(boolean newAbort) {
    abort = newAbort;
  }

  /**
   * 処理を中断するかどうかを取得します。<br/>
   * 
   * @return  処理を中断するかどうか
   */  
  public boolean isAbort() {
    return abort;
  }

  /**
   * 処理を中断した時のエラーメッセージを設定します。<br/>
   * 
   * @param  newAbortMessage  処理中断メッセージ
   */
  public void setAbortMessage(String newAbortMessage) {
    abortMessage = newAbortMessage;
  }

  /**
   * 処理を中断した時のエラーメッセージを取得します。<br/>
   *
   * @return  処理を中断した時のエラーメッセージ
   */  
  public String getAbortMessage() {
    return abortMessage;
  }
  
  /**
   * 永続化で使用されるSQLを設定する。<br/>
   * 
   * @param sql これから実行する（あるいはエラーになった）SQL
   */
  public void setSQL(String sql) {
    this.sql = sql;
  }
  
  /**
   * 永続化で使用されるSQLを取得する。<br/>
   * 
   * @return これから実行する（あるいはエラーになった）SQL
   */
  public String getSQL() {
    return this.sql;
  }

  /**
   * 永続化で使用されるSQLのパラメータを設定する。<br/>
   * 
   * @param params SQLのパラメータ
   */
  public void setParams(Params<?> params) {
    this.params = params;
  }
  
  /**
   * 永続化で使用されるSQLのパラメータを取得する。<br/>
   * 
   * @return SQLのパラメータ
   */
  public Params<?> getParams() {
    return this.params;
  }
}
