/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.event;

import java.io.Serializable;
import java.util.*;


import jp.littlesoft.data.Table;
import jp.littlesoft.data.Row;

/**
 * データテーブル操作イベント。<br/><br/>
 *
 * Tableクラスに対する各種操作が行われた際のイベントオブジェクトです。<br/>
 * このイベントを受け取るには、受け取り側クラスでTableListenerインターフェースを実装し、
 * 対象のデータオブジェクトaddTableListenerで受け取り側オブジェクトを登録します。<br/><br/>
 *
 * @see Table#addTableListener
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class TableEvent<E extends Serializable> extends EventObject implements Serializable {
  
  private Row<E> row;
  private int rowIndex;
  
  /**
   * コンストラクタ。<br/>
   *
   * @param  source  Eventが最初に発生したオブジェクト
   */
  public TableEvent(Object source, Row<E> row, int rowIndex) {
    super(source);
    this.row = row;
    this.rowIndex = rowIndex;
  }
  
  @SuppressWarnings("unchecked")
  @Override
  public Table<E> getSource() {
    // TODO Auto-generated method stub
    return (Table<E>)super.getSource();
  }

  /**
   * イベントが発生した行オブジェクトを取得します。<br/>
   * 
   * @return イベントが発生した行オブジェクト
   */
  public Row<E> getRow() {
    return row;
  }

  /**
   * イベントが発生した行インデックスを取得します。<br/>
   * 
   * @return イベントが発生した行インデックス
   */
  public int getRowIndex() {
    return rowIndex;
  };
  
  /**
   * テーブルオブジェクトに対する操作をしめす列挙型<br/><br/>
   * 
   * <table>
   * <tr><td>START_APPEND</td><td>テーブルオブジェクトで行追加が開始された</td></tr>
   * <tr><td>START_EDIT</td><td>テーブルオブジェクトで行変更が開始された</td></tr>
   * <tr><td>DONE_CANCEL</td><td>テーブルオブジェクトで追加や変更が取り消された</td></tr>
   * <tr><td>DONE_DELETE</td><td>テーブルオブジェクトで行が削除された</td></tr>
   * <tr><td>ROW_DELETING</td><td>テーブルオブジェクトで行が削除されようとしている</td></tr>
   * <tr><td>DONE_VALIDATE</td><td>行の内容確認が行われた</td></tr>
   * <tr><td>DONE_MOVE</td><td>テーブルオブジェクトのカーソル行が移動した</td></tr>
   * <tr><td>DONE_RESOLVE</td><td>テーブルオブジェクトの内容が永続化された</td></tr>
   * <tr><td>DATA_REFRESHED</td><td>テーブルオブジェクトの内容が刷新された</td></tr>
   * <tr><td>DONE_FILTER</td><td>テーブルオブジェクトにフィルターが適用された</td></tr>
   * </table>
   * @author littlesoft
   *
   */
  public enum Operation {START_APPEND,START_EDIT,DONE_CANCEL,DONE_DELETE,ROW_DELETING,
    DONE_VALIDATE,DONE_MOVE,DONE_RESOLVE,DATA_REFRESHED,DONE_FILTER}

}

