/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.event;

import java.io.Serializable;
import java.util.*;

/**
 * データテーブル操作イベントリスナー。<br/><br/>
 *
 * Tableクラスに対する各種操作が行われた際のイベントオブジェクトです。<br/>
 * このイベントを受け取るには、受け取り側クラスでTableListenerインターフェースを実装し、
 * 対象のデータオブジェクトaddTableListenerで受け取り側オブジェクトを登録します。<br/><br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface TableListener extends EventListener {

  /**
   * テーブルオブジェクトのデータが変更された際のイベント。<br/><br/>
   *
   * operationパラメータにはTableEventで定義された定数が設定されます。<br/>
   * 
   * @param  e          TableEvent
   * @param  operation  オペレーション
   * 
   * @see jp.littlesoft.data.event.TableEvent
   */
  public <E extends Serializable> void  dataChanged(TableEvent<?> e, TableEvent.Operation operation);

  /**
   * 行追加、行変更の検証・確定前イベント。<br/><br/>
   *
   * 行単位での値検証等、このイベントに実装します。このメソッドで例外を送出するとテーブルオブジェクトの行に対する変更を中断させることができます。<br/>
   *
   * @param  e  TableEvent
   * @exception  java.lang.Exception  例外発生時
   */
  public <E extends Serializable> void rowValidating(TableEvent<?> e) throws Exception;

  /**
   * 行追加、行変更の削除前イベント。<br/><br/>
   *
   * このメソッドで例外を送出するとテーブルオブジェクトの行削除を中断させることができます。<br/>
   * 
   * @param  e  TableEvent
   * @exception  java.lang.Exception  例外発生時
   */
  public <E extends Serializable> void rowDeleting(TableEvent<?> e) throws Exception;


  /**
   * フィルタ適用時イベント。<br/><br/>
   *
   * このメソッドの戻り値によってテーブルオブジェクトの特定行を不可視にすることができます。<br/>
   *
   * @param  e TableEvent
   * @return boolean true:表示行に含める、false:表示行に含めない。
   */
  public <E extends Serializable> boolean rowFiltering(TableEvent<?> e);

}

