/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.exception;

import jp.littlesoft.data.Column;

/**
 * カラムに関連する例外. <br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class InvalidColumnException extends DataException {

  /** Column */
  private Column column = null;
  private String caption = null;
  private int rowIndex = -1;

  /**
   * コンストラクタ<br/>
   *
   * @param msgResFieldName メッセージリソースのフィールド名
   * @param msgArgs メッセージ中に表示する置換文字列の配列
   * @param column エラーになったカラムオブジェクト
   */
  public InvalidColumnException(String msgResFieldName, Object[] msgArgs, Column column) {
    super(null, null, msgResFieldName, msgArgs);
    this.column = column;
    if (this.column!=null) {
      this.caption = this.column.getCaption();
    }
  }


  /**
   * 例外対象となったカラムオブジェクトを取得します.<br/>
   *
   * @return Column
   */
  public Column getColumn() {
    return this.column;
  }

  /**
   * エラーメッセージに表示するカラムの見出し文字列を設定します.
   *
   * @param caption エラーメッセージに表示するカラムの見出し文字列
   */
  public void setCaption(String caption) {
    this.caption = caption;
  }
  
  /**
   * エラーメッセージに表示するカラム見出し文字列を取得します.
   *
   * @return エラーメッセージに出力するカラム見出し
   */
  public String getCaption() {
    return this.caption;
  }
  
  /**
   * 例外が発生した行インデックスを設定します。
   * 
   * @param rowIndex エラーが発生した行番号
   */
  public void setRowIndex(int rowIndex) {
  	this.rowIndex = rowIndex;
  }
  
  /**
   * 例外が発生した行インデックスを取得します。
   * 
   * (Table.checkNullValues()内で発生した場合のみ、設定されます。)
   * 
   * @return エラーが発生した行番号
   */
  public int getRowIndex() {
  	return this.rowIndex;
  }
}