/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.exception;

import jp.littlesoft.data.Column;
import jp.littlesoft.data.Row;

/**
 * データ検証例外。<br/><br/>
 *
 * データ値の検証時にスローされる例外です。<br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class InvalidValueException extends InvalidColumnException {

  private Object value = null;
  private Row<?> row = null;

  /**
   * コンストラクタ。<br/>
   *
   * @param msgResFieldName メッセージリソースのフィールド名
   * @param msgArgs メッセージ中に表示する置換文字列の配列
   * @param row エラーになった行オブジェクト
   * @param column エラーになったカラム
   * @param value エラーになった値
   */
  public InvalidValueException(String msgResFieldName, Object[] msgArgs, Row<?> row, Column column, Object value) {
    super(msgResFieldName, msgArgs, column);
    this.row = row;
    this.value = value;
  }

  /**
   * 例外となった値を取得します。<br/>
   *
   * @return エラー値
   */
  public Object getValue() {
    return this.value;
  }

  /**
   * 例外が発生した行オブジェクトを返します。<br/>
   * 
   * @return 行オブジェクト
   */
  public Row<?> getRow() {
    return row;
  }
}