/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.soja;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import jp.littlesoft.data.Column;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Column.NoAccessibleObjectException;
import jp.littlesoft.data.exception.InvalidColumnException;
import jp.littlesoft.util.DataUtil;

public class LSJson implements Serializable{

  private static final long serialVersionUID = 6711597448621604711L;
  private HashMap<Class<?>, ITableDef.Columns> serializables = new HashMap<Class<?>, ITableDef.Columns>();
  static private LSJson instance = new LSJson(); 
  
  private LSJson() {}

  private ITableDef.Columns getSerializableElements(Serializable object) {
    return getSerializableElements(object.getClass());
  }
  
  private ITableDef.Columns getSerializableElements(Class<?> clazz) {
    if (!serializables.containsKey(clazz)) {
      ITableDef.Columns columns = new ITableDef.Columns(clazz, false);
      serializables.put(clazz, columns);
    }
    return serializables.get(clazz);
  }
 
  static public Serializable createDummyArray(Class<?> clazz) {
    Class<?> compoType = clazz.getComponentType();
    int dimensions = 1;
    while(compoType.isArray()) {
      dimensions++;
      compoType = compoType.getComponentType();
    }
    int[] d = new int[dimensions];
    for (int i=0; i<dimensions; i++) {
      d[i] = 0;
    }
    return (Serializable)Array.newInstance(compoType, d);
  }
  
  
  @SuppressWarnings("unchecked")
  private Class nameToClass(String className) {
    try {
      return (Class<? extends Serializable>) Class.forName(className);
    } catch (ClassNotFoundException e) {
      throw new RuntimeException(e);
    }
  }
  
  static public String serializeEncoding(Object object) throws JSONException, UnsupportedEncodingException {
    String temp = serialize(object, false);
    //byte[] data = jp.littlesoft.sql4g.Util.compress(temp.getBytes());
    return java.net.URLEncoder.encode(temp, "UTF-8");
  }

  static public String serialize(Object object) throws JSONException {
    return serialize(object, false);
  }
  
  static public String serialize(Object object, boolean indent) throws JSONException {
    JSONObject json = instance.toJSONObject(object);
    if (indent){
      return json.toString(2);
    } else {
      return json.toString();
    }
  }
  
  boolean isSimpleType(Object value) {
    return (value == null
         || value instanceof Boolean
         || value instanceof String
         || value instanceof Number
         || value instanceof java.util.Date);
  }
  
  private void toJSONArrays(ArrayList<Object> array, Object[] values) throws JSONException {
    for (Object value : values) {
      if (value == null) {
        array.add(null);
      } else if (value.getClass().isArray()) {
        ArrayList<Object> children = new ArrayList<Object>();
        toJSONArrays(children, (Object[])value);
        array.add(children);
      } else if (isSimpleType(value)) {
        array.add(value);
      } else {
        array.add(toJSONObject((Serializable)value));
      }
    }
    //return jsonArray;
  }
  
  private JSONObject toJSONObject(Object object) throws JSONException {
    JSONObject json = new JSONObject();
    json.put("class", object.getClass().getName());
    if (object.getClass().isArray()) {
      Class<?> compoType = object.getClass().getComponentType();
      String className = "[]";
      while (compoType.isArray()) {
        className += "[]";
        compoType = compoType.getComponentType();
      }
      className = compoType.getName() + className;
      ArrayList<Object> array = new ArrayList<Object>();
      toJSONArrays(array, (Object[])object);
      json.put("values", new JSONArray(array));  
      json.put("class", className);
      
    } else if (object instanceof Collection) {
      ArrayList<Object> array = new ArrayList<Object>();
      for (Object value : ((Collection<?>)object)) {
        if (isSimpleType(value)) {
          array.add(value);
        } else {
          array.add(toJSONObject((Serializable)value));
        }
      }      
      json.put("values", new JSONArray(array));  
    } else if (object instanceof Map) {
      Map<?, ?> objectMap = (Map<?, ?>)object;
      HashMap<Object, Object> jsonMap = new HashMap<Object, Object>();
      for (Object key : objectMap.keySet()) {
        Object jsonKey = key;
        if (!isSimpleType(key)) {
          jsonKey = toJSONObject((Serializable)jsonKey);
        }
        Object jsonValue = objectMap.get(key);
        if (!isSimpleType(jsonValue)) {
          jsonValue = toJSONObject((Serializable)jsonValue);
        }
        jsonMap.put(jsonKey, jsonValue);
      }
      json.put("values", jsonMap);  
    } else {
      String name;
      for (Column element : getSerializableElements((Serializable)object)) {
        if (element.isReadOnly()) continue;
        name = element.getName();
        Object value = null;
        try {
          value = element.getEntityValue((Serializable)object);
        } catch (NoAccessibleObjectException e) {
          e.printStackTrace();
        }
        if (value == null) {
          json.put(name, JSONObject.NULL);
          continue;
        }

        switch (element.getType()) {
        case STRING:
        case CLOB:
          json.put(name, (String) value);
          break;
        case BOOLEAN:
          json.put(name, ((Boolean) value).booleanValue());
          break;
        case BYTE:
        case SHORT:
        case INT:
          json.put(name, ((Number) value).intValue());
          break;
        case LONG:
          json.put(name, ((Number) value).longValue());
          break;
        case DOUBLE:
        case FLOAT:
        case BIGDECIMAL:
          json.put(name, ((Number) value).doubleValue());
          break;
        case DATE:
        case TIME:
        case TIMESTAMP:
          if (!element.getFormatPattern().equals("")) {
            try {
              json.put(name, DataUtil.formatDate(element, (java.util.Date)value));
            } catch (InvalidColumnException e) {
              throw new RuntimeException(e);
            }
          } else {
            json.put(name, value.toString());
          }
          break;
        default:
          json.put(name, toJSONObject((Serializable)value));
        }
      }
    }
    return json;
  }
  
  static public Serializable deserializeDecoding(String jsonString) throws JSONException, UnsupportedEncodingException {
    String temp = java.net.URLDecoder.decode(jsonString,"UTF-8");
    //byte[] data = jp.littlesoft.sql4g.Util.expand(temp.getBytes(),1);
    return deserialize(temp);
  }

  static public Serializable deserialize(String jsonString) throws JSONException {
    if (jsonString == null) return null;
    return instance.toJavaObject(new JSONObject(jsonString));
  }
  
  private void checkDimensions(JSONArray jsonArray, int depth, ArrayList<Integer> dims) throws JSONException{
    while (dims.size()<(depth+1)) dims.add(0);
    for (int i=0; i<jsonArray.length(); i++) {
      if (jsonArray.get(i) instanceof JSONArray) {
        checkDimensions(jsonArray.getJSONArray(i), depth + 1,  dims);
      } else if (jsonArray.get(i).getClass().isArray()) {
        //System.out.println("Array : "  + jsonArray.get(i).toString());
      } else {
        //System.out.println("What? :  " + jsonArray.get(i).toString());
      }
    }
    if (dims.get(depth)<jsonArray.length()) dims.set(depth, jsonArray.length());
  }
  
  private void setDimensions(JSONArray jsonArray, Object array[], int depth, int[] dims) throws JSONException{
    for (int i=0; i<dims[depth]; i++) {
      if (jsonArray.get(i) instanceof JSONArray) {
        setDimensions(jsonArray.getJSONArray(i), (Object[])array[i], depth + 1,  dims);
      } else {
        array[i] = jsonArray.get(i);
      }
    }
  }

  @SuppressWarnings("unchecked")
  private Serializable toJavaObject(JSONObject json) throws JSONException {
    String className = json.getString("class").trim();
    if (className.endsWith("]")) {
      className = className.substring(0, className.indexOf('['));
      JSONArray jsonArray = json.getJSONArray("values");
      ArrayList<Integer> dims = new ArrayList<Integer>();
      checkDimensions(jsonArray, 0, dims);
      int[] dimensions = new int[dims.size()];
      int i = 0;
      for (Integer d : dims) {
        dimensions[i++] = d;
      }
      Object array = null;
      try {
        array = Array.newInstance(Class.forName(className), dimensions);
        setDimensions(jsonArray, (Object[])array, 0, dimensions);
      } catch (Exception e) {
        throw new JSONException(e);
      }
      return (Serializable)array;
    }
    
    Class<?> clazz = nameToClass(json.getString("class"));
    if (clazz==null) {
      return null;
    } else if (Collection.class.isAssignableFrom(clazz)) {
      ArrayList<Serializable> list = new ArrayList<Serializable>();
      JSONArray values = (JSONArray)json.get("values");
      if (values!=null) {
        for (int i = 0; i < values.length(); i++) {
          if (JSONObject.NULL.equals(values.get(i))) {
            list.add(null);
          } else if (isSimpleType(values.get(i))) {
            list.add((Serializable) values.get(i));
          } else {
            list.add(toJavaObject((JSONObject)values.get(i)));
          }
        }
      }
      return list;
    } else if (Map.class.isAssignableFrom(clazz)) {
      HashMap<String, Object> map = null;
      try {
        Object obj = clazz.newInstance();
        map = (HashMap<String, Object>)obj;
      } catch (Exception x) {
        map = new HashMap<String, Object>();
      }
      JSONObject values = (JSONObject)json.get("values");
      String[] names = null;
      if ( values!=null && (names = JSONObject.getNames(values))!=null ) {
        for (String key : names) {
          Object value = values.get(key);
          if (JSONObject.NULL.equals(value)) {
            value = null;
          } else if (!isSimpleType(value)) {
            value = toJavaObject((JSONObject)value);
          }
          map.put(key, value);
        }
      }
      return map;
    } else {
      Serializable entity = null;
      try {
        entity = (Serializable)clazz.newInstance();
        for (Column element : getSerializableElements(clazz)) {
          String name = element.getName();
          if (element.isReadOnly() || json.isNull(name)) continue;
          switch(element.getType()) {
          case STRING:
          case CLOB:
            element.setEntityValue(entity, (Serializable)json.get(name));
            break;
          case BOOLEAN:
            element.setEntityValue(entity, json.getBoolean(name));
            break;            
          case BYTE:
            element.setEntityValue(entity, (byte)json.getInt(name));
            break;            
          case SHORT:
            element.setEntityValue(entity, (short)json.getInt(name));
            break;            
          case INT:
            element.setEntityValue(entity, json.getInt(name));
            break;            
          case LONG:
            element.setEntityValue(entity, json.getLong(name));
            break;            
          case DOUBLE:
            element.setEntityValue(entity, json.getDouble(name));
            break;            
          case FLOAT:
            element.setEntityValue(entity, new Float(json.getDouble(name)));
            break;            
          case BIGDECIMAL:
            element.setEntityValue(entity, new BigDecimal(json.getDouble(name)));
            break;            
          case DATE:
            if (!element.getFormatPattern().equals("")) {
              element.setEntityValue(entity, (Serializable)DataUtil.parseObject(element, (String)json.get(name)));
            } else {
              element.setEntityValue(entity, java.sql.Date.valueOf((String)json.get(name)));
            }
            break;            
          case TIME:
            if (!element.getFormatPattern().equals("")) {
              element.setEntityValue(entity, (Serializable)DataUtil.parseObject(element, (String)json.get(name)));
            } else {
              element.setEntityValue(entity, Time.valueOf((String)json.get(name)));
            }
            break;            
          case TIMESTAMP:
            if (!element.getFormatPattern().equals("")) {
              element.setEntityValue(entity, (Serializable)DataUtil.parseObject(element, (String)json.get(name)));
            } else {
              element.setEntityValue(entity, Timestamp.valueOf((String)json.get(name)));
            }
            break;            
          default:
            element.setEntityValue(entity, toJavaObject(json.getJSONObject(name)));
          }
        }  
      } catch (Exception e) {
        throw new JSONException(e);
      }
      return entity;
    }
  }

  
  /**
   * @param args
   */
  public static void main(String[] args) {
    // TODO Auto-generated method stub

  }

  @Override
  public String toString() {
    // TODO Auto-generated method stub
    return super.toString();
  }

}