/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.soja;

import java.io.Serializable;
import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.NClob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.RowIdLifetime;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Savepoint;
import java.sql.Statement;
import java.sql.Struct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStreamWriter;

import org.json.JSONException;
import jp.littlesoft.data.Column;
import jp.littlesoft.data.Params;
import jp.littlesoft.data.Row;
import jp.littlesoft.data.Table;
import jp.littlesoft.data.event.ResolvingListener;
import jp.littlesoft.data.exception.DataException;
import jp.littlesoft.soja.LSJson;
import jp.littlesoft.sql.ISQLDataProvider;
import jp.littlesoft.sql.ISQLDataResolver;
import jp.littlesoft.sql.ISQLTableStorage;
import jp.littlesoft.sql.logger.SQLLogger;
import jp.littlesoft.sql.logger.SQLTimeLogger;
import jp.littlesoft.util.StringUtil;

@SuppressWarnings("serial")
public class LSJsonConnection implements Connection, Serializable, ISQLTableStorage {
  
  static public String REQUEST_PARAM_NAME = Integer.toHexString(Request.class.getName().hashCode());
  
  static public class Provider implements ISQLDataProvider {
    private LSJsonConnection conn;
    private String sql;
    private Params<?> params;
    private int maxRows = 0; 
    private String clientId;
    //private HttpURLConnection connection;
    private Response response = null;
    
    public Provider(LSJsonConnection conn) {
      this.conn = conn;
      clientId = String.valueOf(this.hashCode());
    }
    
    @Override
    public void close() throws DataException {}

    @Override
    public Connection getConnection() {
      return conn;
    }

    @Override
    public void setConnection(Connection conn) throws DataException {
      this.conn = (LSJsonConnection)conn;
    }
    @Override
    public String getSQL() {
      return sql;
    }
    @Override
    public void setSQL(String sql) throws DataException {
      this.sql = sql;
    }
    @Override
    public Params<?> getParams() {
      return params;
    }
    @Override
    public void setParams(Params<?> params) {
      this.params = params;
    }
    
    @Override
    public void setMaxRows(int newMaxRows) {
      maxRows = newMaxRows;
    }

    private Map<String, Object> getParamsAsMap() throws DataException {
      HashMap<String, Object> map = new HashMap<String, Object>();
      for (int i=0; i<getParams().getColumnCount(); i++) {
        if (getParams().getColumn(i).isReadOnly() ||
            getParams().isNull(i)) continue;
        map.put(getParams().getColumn(i).getName(), getParams().getValue(i));
      }
      return map;
    }
    
    @Override
    public <E extends Serializable> void setupColumn(Table<?> table) throws DataException {
      if (!table.isTypedRow()) throw new DataException("Remote database query must be typed using the table.");
      for (Column col: table.getColumns().values() ) {
        col.setFromDatabase(false);
      }
      //照会時間のログ出力
      long start = System.currentTimeMillis();
      try {
        if (StringUtil.isNullString(this.getSQL())) {
          throw new DataException(null, null, "SQL_IS_NULL", new String[]{""});
        }
        // 照会ログ出力
        if (SQLLogger.isQueryDebugEnabled()) {
          SQLLogger.logForQuery(clientId, getSQL(), this.getParams());
        }
        Request request = new Request(conn.database, conn.user, conn.password,
            table.getEntityType().getName(), getSQL(), maxRows, getParamsAsMap()) ;
          table.setAllRowCount(-1);
        
        /*  
        String paramString = "?request=" + URLEncoder.encode(LSJson.serialize(request), "UTF-8");
        URL url = new URL(conn.url + paramString);
        BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream(),"UTF-8"));
        String line;
        java.io.StringWriter sw = new java.io.StringWriter();
        while ((line = reader.readLine()) != null) {
          sw.append(line);
        }
        response = (Response)LSJson.deserialize(sw.toString());
        reader.close();
        if (response==null) {
          throw new DataException("Empty response from the remote database server.");
        } else if (response.hasError) {
          throw new DataException("An exception occurred inside the remote database server:" 
              + response.entytyTypeName + "[" + response.message + "]");
        }
        */  
        URL url = new URL(conn.url);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setDoOutput(true);
        connection.setRequestMethod("POST");
        OutputStreamWriter writer = new OutputStreamWriter(connection.getOutputStream());
        writer.write(REQUEST_PARAM_NAME + "=" + URLEncoder.encode(LSJson.serialize(request), "UTF-8"));
        writer.close();
        java.io.StringWriter sw = new java.io.StringWriter();
        if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
          InputStreamReader ir = new InputStreamReader(connection.getInputStream(),"UTF-8");
          BufferedReader reader = new BufferedReader(ir); 
          String line;
          while ((line = reader.readLine()) != null) {
            sw.append(line);
          }
          response = (Response)LSJson.deserialize(sw.toString());
          reader.close();
          if (response==null) {
            throw new DataException("Empty response from the remote database server.");
          } else if (response.hasError) {
            throw new DataException("An exception occurred inside the remote database server:" 
                + response.entytyTypeName + "[" + response.message + "]");
          }
        } else {
          throw new DataException("Server returned HTTP error code[" + connection.getResponseCode() + "] URL[" + conn.url + "]"); 
        }
      } catch (MalformedURLException e) {
        throw new DataException(e);
      } catch (IOException e) {
        throw new DataException(e);
      } catch (JSONException e) {
        throw new DataException(e);
      } finally {
        // ---- 照会時間のログ出力
        if (SQLTimeLogger.isQueryDebugEnabled()) {
          long provideTime = System.currentTimeMillis() - start;
          String qaName = "";
          SQLTimeLogger.logForQuery(SQLTimeLogger.PROVIDE, clientId, qaName, "", provideTime);
        }
        //connection.disconnect();
        //connection = null;
      }
    }

    @SuppressWarnings("unchecked")
    @Override
    public void loadData(Table table) throws DataException {
      internalLoadData(table);
    }

    @SuppressWarnings("unchecked")
    public <T extends Serializable> void internalLoadData(Table<T> table) throws DataException {
      List<T> resultSet = (List<T>) response.result;
      for (T entity : resultSet){
        Row<T> newRow = new Row<T>(table, entity);
        newRow.setStatus(Row.Status.LOADED);
        table.internalPut(newRow, true, false);
      }
    }

    @Override
    public String getPageSQL() { throw new UnsupportedOperationException();  }
    @Override
    public void setPageSQL(String sql, int i, int showRows) throws DataException { throw new UnsupportedOperationException(); }
    @Override
    public boolean checkMoreRow(Table<?> table, boolean toLast) throws DataException { return false; }
    @Override
    public void free() throws DataException {}
  }
  
  
  static public class Resolver implements ISQLDataResolver {
    public Resolver() {
      
    }

    @Override
    public void close() throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public Connection getConnection() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String[] getKeyColumns() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getSQLDelete() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getSQLInsert() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getSQLUpdate() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getTargetDBCatalog() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getTargetDBSchema() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getTargetDBTable() {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public void setConnection(Connection conn) throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setKeyColumns(String[] newResolveKeyColumns) throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setPendingCommit(boolean pendingCommit) {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setSQLDelete(String sqlDelete) {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setSQLInsert(String sqlInsert) {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setSQLUpdate(String sqlUpdate) {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setTargetDBCatalog(String targetDBCatalog) throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setTargetDBSchema(String targetDBSchema) throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void setTargetDBTable(String targetDBTable) throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void addResolvingListener(ResolvingListener l) {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void execute(Table<?> dataTable) throws DataException {
      // TODO Auto-generated method stub
      
    }

    @Override
    public void removeResolvingListener(ResolvingListener l) {
      // TODO Auto-generated method stub
      
    }
  }

  /**
   * 
   * @author littlesoft
   *
   */
  static public class Request implements Serializable{
    
    public Request() {};
    
    public Request(String database, String user, String password, 
                   String entytyTypeName, String sql, Integer maxRows, Map<String, Object> params) {
      this.database = database;
      this.entytyTypeName = entytyTypeName;
      this.user = user;
      this.password = password;
      this.sql = sql;
      this.params = params;
    }
    
    public String database;
    public String user;
    public String password;
    public String sql;
    public Integer maxRows; 
    public String entytyTypeName;
    public Map<String, Object> params;
  }

  /**
   * 
   * @author littlesoft
   *
   */
  static public class Response extends Request{
    public Response() {}
    
    public void setRequest(Request reuest) {
      this.database = reuest.database;
      this.user = reuest.user;
      this.sql = reuest.sql;
      this.password = reuest.password;
      this.params = reuest.params;
    }
    public List<?> result;
    public Boolean hasError = new Boolean(false);
    public String exceptionTypeName;
    public String message;
  }
  
  String url;
  String database;
  String user;
  String password;
  
  /**
   * 
   * @param url
   * @param database
   * @param user
   * @param password
   */
  public LSJsonConnection(String url, String database, String user, String password) {
    this.url = url;
    this.database = database;
    this.user = user;
    this.password = password;
  }  
  
  @Override
  public ISQLDataProvider getProvider() {
    return new LSJsonConnection.Provider(this);
  }
  
  @Override
  public ISQLDataResolver getResolver() {
    return new LSJsonConnection.Resolver();
  }
  
  @Override
  public void clearWarnings() throws SQLException {}

  @Override
  public void close() throws SQLException {}

  @Override
  public void commit() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Blob createBlob() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Clob createClob() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public NClob createNClob() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public SQLXML createSQLXML() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Statement createStatement() throws SQLException {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public boolean getAutoCommit() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public String getCatalog() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Properties getClientInfo() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public String getClientInfo(String name) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public int getHoldability() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public DatabaseMetaData getMetaData() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public int getTransactionIsolation() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Map<String, Class<?>> getTypeMap() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public SQLWarning getWarnings() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public boolean isClosed() throws SQLException {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public boolean isReadOnly() throws SQLException {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public boolean isValid(int timeout) throws SQLException {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public String nativeSQL(String sql) throws SQLException {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public CallableStatement prepareCall(String sql) throws SQLException {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
      throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public PreparedStatement prepareStatement(String sql) throws SQLException {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
      throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public void releaseSavepoint(Savepoint savepoint) throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public void rollback() throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public void rollback(Savepoint savepoint) throws SQLException {
    throw new UnsupportedOperationException(); 
  }
  @Override
  public void setAutoCommit(boolean autoCommit) throws SQLException {
    //
  }
  @Override
  public void setCatalog(String catalog) throws SQLException {
    // TODO Auto-generated method stub

  }
  public void setClientInfo(Properties properties)  {}

  public void setClientInfo(String name, String value) {}

  @Override
  public void setHoldability(int holdability) throws SQLException {}

  @Override
  public void setReadOnly(boolean readOnly) throws SQLException {}

  @Override
  public Savepoint setSavepoint() throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public Savepoint setSavepoint(String name) throws SQLException {
    throw new UnsupportedOperationException(); 
  }

  @Override
  public void setTransactionIsolation(int level) throws SQLException {}

  @Override
  public void setTypeMap(Map<String, Class<?>> map) throws SQLException {}

  @Override
  public boolean isWrapperFor(Class<?> iface) throws SQLException {
    return false;
  }

  @Override
  public <T> T unwrap(Class<T> iface) throws SQLException {
    return null;
  }

  public class JDBCMetaData implements DatabaseMetaData {

    @Override
    public boolean allProceduresAreCallable() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean allTablesAreSelectable() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean autoCommitFailureClosesAllResultSets() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean dataDefinitionCausesTransactionCommit() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean dataDefinitionIgnoredInTransactions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean deletesAreDetected(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean doesMaxRowSizeIncludeBlobs() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public ResultSet getAttributes(String catalog, String schemaPattern, String typeNamePattern, String attributeNamePattern)
        throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getBestRowIdentifier(String catalog, String schema, String table, int scope, boolean nullable)
        throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getCatalogSeparator() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getCatalogTerm() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getCatalogs() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getClientInfoProperties() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getColumnPrivileges(String catalog, String schema, String table, String columnNamePattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getColumns(String catalog, String schemaPattern, String tableNamePattern, String columnNamePattern)
        throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public Connection getConnection() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getCrossReference(String parentCatalog, String parentSchema, String parentTable, String foreignCatalog,
        String foreignSchema, String foreignTable) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public int getDatabaseMajorVersion() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getDatabaseMinorVersion() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public String getDatabaseProductName() throws SQLException {
      return null;
    }

    @Override
    public String getDatabaseProductVersion() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public int getDefaultTransactionIsolation() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getDriverMajorVersion() {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getDriverMinorVersion() {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public String getDriverName() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getDriverVersion() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getExportedKeys(String catalog, String schema, String table) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getExtraNameCharacters() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getFunctionColumns(String catalog, String schemaPattern, String functionNamePattern, String columnNamePattern)
        throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getFunctions(String catalog, String schemaPattern, String functionNamePattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getIdentifierQuoteString() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getImportedKeys(String catalog, String schema, String table) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getIndexInfo(String catalog, String schema, String table, boolean unique, boolean approximate)
        throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public int getJDBCMajorVersion() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getJDBCMinorVersion() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxBinaryLiteralLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxCatalogNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxCharLiteralLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxColumnNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxColumnsInGroupBy() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxColumnsInIndex() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxColumnsInOrderBy() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxColumnsInSelect() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxColumnsInTable() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxConnections() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxCursorNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxIndexLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxProcedureNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxRowSize() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxSchemaNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxStatementLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxStatements() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxTableNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxTablesInSelect() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public int getMaxUserNameLength() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public String getNumericFunctions() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getPrimaryKeys(String catalog, String schema, String table) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getProcedureColumns(String catalog, String schemaPattern, String procedureNamePattern, String columnNamePattern)
        throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getProcedureTerm() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getProcedures(String catalog, String schemaPattern, String procedureNamePattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public int getResultSetHoldability() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public RowIdLifetime getRowIdLifetime() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getSQLKeywords() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public int getSQLStateType() throws SQLException {
      // TODO Auto-generated method stub
      return 0;
    }

    @Override
    public String getSchemaTerm() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getSchemas() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getSchemas(String catalog, String schemaPattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getSearchStringEscape() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getStringFunctions() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getSuperTables(String catalog, String schemaPattern, String tableNamePattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getSuperTypes(String catalog, String schemaPattern, String typeNamePattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getSystemFunctions() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getTablePrivileges(String catalog, String schemaPattern, String tableNamePattern) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getTableTypes() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getTables(String catalog, String schemaPattern, String tableNamePattern, String[] types) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getTimeDateFunctions() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getTypeInfo() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getUDTs(String catalog, String schemaPattern, String typeNamePattern, int[] types) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getURL() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getUserName() throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public ResultSet getVersionColumns(String catalog, String schema, String table) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public boolean insertsAreDetected(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean isCatalogAtStart() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean isReadOnly() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean locatorsUpdateCopy() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean nullPlusNonNullIsNull() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean nullsAreSortedAtEnd() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean nullsAreSortedAtStart() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean nullsAreSortedHigh() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean nullsAreSortedLow() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean othersDeletesAreVisible(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean othersInsertsAreVisible(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean othersUpdatesAreVisible(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean ownDeletesAreVisible(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean ownInsertsAreVisible(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean ownUpdatesAreVisible(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean storesLowerCaseIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean storesLowerCaseQuotedIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean storesMixedCaseIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean storesMixedCaseQuotedIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean storesUpperCaseIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean storesUpperCaseQuotedIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsANSI92EntryLevelSQL() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsANSI92FullSQL() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsANSI92IntermediateSQL() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsAlterTableWithAddColumn() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsAlterTableWithDropColumn() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsBatchUpdates() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCatalogsInDataManipulation() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCatalogsInIndexDefinitions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCatalogsInPrivilegeDefinitions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCatalogsInProcedureCalls() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCatalogsInTableDefinitions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsColumnAliasing() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsConvert() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsConvert(int fromType, int toType) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCoreSQLGrammar() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsCorrelatedSubqueries() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsDataDefinitionAndDataManipulationTransactions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsDataManipulationTransactionsOnly() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsDifferentTableCorrelationNames() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsExpressionsInOrderBy() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsExtendedSQLGrammar() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsFullOuterJoins() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsGetGeneratedKeys() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsGroupBy() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsGroupByBeyondSelect() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsGroupByUnrelated() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsIntegrityEnhancementFacility() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsLikeEscapeClause() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsLimitedOuterJoins() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsMinimumSQLGrammar() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsMixedCaseIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsMixedCaseQuotedIdentifiers() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsMultipleOpenResults() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsMultipleResultSets() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsMultipleTransactions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsNamedParameters() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsNonNullableColumns() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsOpenCursorsAcrossCommit() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsOpenCursorsAcrossRollback() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsOpenStatementsAcrossCommit() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsOpenStatementsAcrossRollback() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsOrderByUnrelated() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsOuterJoins() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsPositionedDelete() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsPositionedUpdate() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsResultSetConcurrency(int type, int concurrency) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsResultSetHoldability(int holdability) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsResultSetType(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSavepoints() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSchemasInDataManipulation() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSchemasInIndexDefinitions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSchemasInPrivilegeDefinitions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSchemasInProcedureCalls() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSchemasInTableDefinitions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSelectForUpdate() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsStatementPooling() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsStoredFunctionsUsingCallSyntax() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsStoredProcedures() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSubqueriesInComparisons() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSubqueriesInExists() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSubqueriesInIns() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsSubqueriesInQuantifieds() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsTableCorrelationNames() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsTransactionIsolationLevel(int level) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsTransactions() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsUnion() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean supportsUnionAll() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean updatesAreDetected(int type) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean usesLocalFilePerTable() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean usesLocalFiles() throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public boolean isWrapperFor(Class<?> arg0) throws SQLException {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public <T> T unwrap(Class<T> arg0) throws SQLException {
      // TODO Auto-generated method stub
      return null;
    }
    
  }

}
