/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.sql;

import java.io.Serializable;
import java.sql.Connection;

import jp.littlesoft.data.Column;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.sql.SQL.BindParameter;
import jp.littlesoft.sql.SQL.Comparer;
import jp.littlesoft.sql.SQL.Condition;
import jp.littlesoft.sql.SQL.Delete;
import jp.littlesoft.sql.SQL.Expression;
import jp.littlesoft.sql.SQL.Insert;
import jp.littlesoft.sql.SQL.Select;
import jp.littlesoft.sql.SQL.Update;

/**
 * SQL対応アプリケーション用 インターフェース.<br/>
 * アプリケーションプログラムから、jp.littlesoft.sql.SQLの機能を呼び出す為のインターフェース
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface ISQLApplication {


  
  /**
   * ダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   */
  <E extends Serializable> E $useDummy(Class<? extends E> type);
  
  /**
   * リモートデータベースに接続するダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   * @param id   リモートデータベースを識別する任意のID
   */
  <E extends Serializable> E $useRemoteDummy(Class<? extends E> type, String id);

  /**
   * ダミーエンティティオブジェクトにリモートデータベースのIDを割り当てます.<br/>
   * @param dummy  ダミーエンティティオブジェクﾄ
   * @param id  リモートデータベースを識別する任意のID
   */
  void $useRemote(ITableDef dummy, String id); 
  /**
   * SELECTオブジェクトを作成します。.<br/>
   * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
   * 省略すると[SELECT * FROM ...] と同じ意味になります。
   * @param columns ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
   */
  Select select(Serializable...columns);
  
  /**
   * バインドパラメータを作成します.<br/>
   * @param パラメータ名
   */
  BindParameter $bindParam(String name);
  
  /**
   * 条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  @SuppressWarnings("unchecked")
  Condition<?>  $if(Condition nestedCondition) ;

  /**
   * 条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  Comparer<?, ?>   $if(Serializable value);
  
  /**
   * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  Condition<?> $evaluateNonNulls(Condition<?> nestedCondition);
  
  /**
   * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  Comparer<?, ?> $evaluateNonNulls(Serializable value);
  /**
   * SQL式を開始します.<br/>
   * @param source ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
   */
  Expression $(Serializable source);
  
  /**
   * ダミーエンティティオブジェクトのフィールドから対応するカラムオブジェクトを取得します.<br/>
   */
  Column $column(Serializable source);
  
  /**
   * DELETE文を構築します.<br/>
   * @param entity
   * @return Delete
   */
  Delete delete(Serializable entity); 
  
  /**
   * INSERT文を構築します.<br/>
   * @param entity
   * @return Insert
   */
  Insert insert(Serializable entity); 

  /**
   * UPDATE文を構築します.<br/>
   * @param entity
   * @return Update
   */
  Update update(Serializable entity); 
  
  /**
   * キーを指定してダイナミック条件式を設定します.<br/>
   * @param key 条件式を識別するキー
   */
  void setDynamicCondtion(String key, Condition<?> condition);
  
  /**
   * 設定済のダイナミック条件式を全てクリアします。
   */
  void resetDynamicCondtions();
  
  /**
   * リモートデータベースのコネクションを設定します.
   * @param id リモートデータベースを識別する任意のID
   * @param connection JDBCコネクション
   */
  void setRemoteConnection(String id, Connection connection);
  

}
