/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.sql;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.List;

import jp.littlesoft.data.Column;
import jp.littlesoft.data.EntityList;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Params;
import jp.littlesoft.data.Row;
import jp.littlesoft.data.Table;
import jp.littlesoft.data.ITableDef.MetaData;
import jp.littlesoft.data.exception.DataException;
import jp.littlesoft.soja.LSJson;
import jp.littlesoft.soja.LSJsonConnection;
import jp.littlesoft.sql.SQLAdaptor.SQLServerType;
import jp.littlesoft.sql.exception.LSSQLException;
import jp.littlesoft.web.WebSQLTable;

@SuppressWarnings({ "unchecked", "serial" })
public class SQL<T extends Serializable> implements Serializable {
  
  Query query = null;
  Class entityType;
  Params<T> _params = null;
  T[] typeParam = null;
  Class dataProviderType = null;
  Class dataResolverType = null;
  HashMap<String, Remote> idToRemote;
  HashMap<ITableDef, Remote> entityToRemote = null;
  
  HashMap<ITableDef, Remote> getEntityToRemote() {
    return entityToRemote;
  }
  
  static public class Remote implements Serializable {
    String id;
    Connection connection;
    ArrayList<ITableDef> entiies = new ArrayList<ITableDef>();
    HashMap<ITableDef, String> sqls = new HashMap<ITableDef, String>();
    public Remote(String id) {
      this.id = id;
    }
    
    public Connection getConnection() {
      return connection;
    }
    
    public void setConnection(Connection connection) {
      this.connection = connection;
    }
    
    public ArrayList<ITableDef> getEntites() {
      return entiies;
    }
    
    public HashMap<ITableDef, String> getQueries() {
      return sqls;
    }
  }    

  public void setRemoteConnection(String id, Connection connection) {
    idToRemote.get(id).setConnection(connection);
  }
  
  
  
  /**
   * このSQLのターゲットエンティティのダミーオブジェクト.<br/>
   * SQL文を作成する際使用します.
   */
  public T $$ = null;
  
  /**
   * このSQLのターゲットエンティティのパラメータオブジェクト.<br/>
   * this.getParams().entity または this.getParams().getEntity()と同じオブジェクトです.<br/>
   */
  public T params = null;
  
  /**
   * コンストラクタ
   * SQLターゲットエンティティのクラスをパラメータで指定します.<br/>
   * SQL&lt;Test&gt; test = new SQL&lt;Test&gt;();
   * @param e
   */
  public SQL(T... e) {
    entityType = (Class<T>) e.getClass().getComponentType();
    typeParam = e;
    //setParams(new Params<T>(typeParam));
    try {
      if (!entityType.equals(Object.class)) {
        $$ = (T) $useDummy(entityType);
      }
    } catch (Exception x) {}
  }

  public SQL(WebSQLTable<T> table, T... e) {
    entityType = (Class<T>) e.getClass().getComponentType();
    typeParam = e;
    if (table.$$==null) return;
    $$ = table.$$;
    _useDummy(table.$$, null);
  }
  /**
   * SQLのパラメータを外部から設定します.<br/>
   * @param param
   */
  public void setParams(Params<T> param) {
    _params = param;
    if (_params.isTypedRow()) {
      params = _params.getEntity();
    }
  }
  
  /**
   * SQLのパラメータを取得します.<br/>
   * @return Params
   */
  public Params<T> getParams() {
    if (_params == null) {
      setParams(new Params<T>(typeParam));
    }
    return _params;
  }
  
  public String getSQL() {
    return query.getSQL(false);
  }
  
  /**
   * SQLのパラメータをの値をすべてクリアします.<br/>
   */
  public void clearParams() {
    try {
      getParams().clearColumns();
    } catch (DataException e) {
      throw new RuntimeException(e);
    }
  }
  
  /**
   * このSQLオブジェクトで生成された内容をすべてリセットします.<br/>
   * ひとつのSQLオブジェクトをselect,delete,insert,updateメソッドで再利用する場合はresetAllで初期化する必要があります。<br/>
   */
  public void resetAll() {
    clearParams();
    query = null;
    _params = null;
  }

  public MetaData getMetaData(Serializable dummyEntity) {
    if (this.query!=null) {
      return this.query.getMetaData(dummyEntity);
    } else {
      return null;
    }
  }
  
  void setQuery(Query query) {
    this.query = query;
    this.query.setSQLContext(this);
  }
  
  /**
   * SELECT以外のSQLを実行します.<br/>
   * @param conn JDBCコネクション
   * @param pendingCommit true:コミットを保留する。(AutoCommitしない)
   * @param reuseStatement true:JDBCのステートメントを再利用する
   * @throws DataException
   */
  public int execute(Connection conn, boolean pendingCommit, boolean reuseStatement) throws DataException {
    return query.execute(conn, pendingCommit, reuseStatement);
  }
  
  /**
   * SELECTを実行し、結果リストを返します.<br/>
   * @param conn JDBCコネクション
   * @return List<T> 結果エンティティのリスト
   * @throws DataException
   */
  public List<T> execute(Connection conn)  throws DataException {
    if (!(query instanceof Select)) {
      throw new DataException("This operation is invalid.");
    } 
    return ((Select)query).execute(conn, typeParam, _params);
  }
  
  /**
   * SELECTを実行し、テーブルオブジェクトを返します.<br/>
   * @param conn JDBCコネクション
   * @return Table<T> 結果が格納されたテーブルオブジェクト
   * @throws DataException
   */
  public Table<T> getActiveTable(Connection conn)  throws DataException {
    if (!(query instanceof Select)) {
      throw new DataException("This operation is invalid.");
    } 
    return ((Select)query).getTable(conn, typeParam, _params);
  }
  
  /**
   * SQLとパラメータ内容を文字列として取得します.<br/>
   */
  @Override
  public String toString() {
    if (query!=null) {
      query.includeNullParamCondition = true;
      try {
        return this.getClass().getName() +  "{\n" + query.createSQL() + "}\n" + getParams().toString();
      } finally {
        query.includeNullParamCondition = false;
      }
    }
    return "";
  }
  
  /**
   * 駱駝スタイルのnameをアンダースコア区切りに変換する.
   * @param name
   * @param changeAllCharToUpperCase true:小文字を全て大文字にする false:大文字を全て小文字にする。
   * @return
   */
  public static String camelToUnderscore(String name , boolean changeAllCharToUpperCase) {
    StringBuffer sBuff = new StringBuffer();
    char[] nameChars = name.toCharArray();
    for (int i=0; i<nameChars.length; i++) {
      if (i>0 && Character.isUpperCase(nameChars[i]) && nameChars[i-1]!='_') {
        sBuff.append('_');
        sBuff.append((changeAllCharToUpperCase) ? nameChars[i] : Character.toLowerCase(nameChars[i]));
      } else {
        sBuff.append((changeAllCharToUpperCase) ? Character.toUpperCase(nameChars[i]) : Character.toLowerCase(nameChars[i]));
      }
    }
    return sBuff.toString();
  }
  
  /**
   * アンダースコア区切りのnameを駱駝スタイルに変換する.
   * @param name
   * @param changeInitialCharToUpperCase 先頭文字を大文字にする。
   * @return
   */
  public static String underscoreToCamel(String name, boolean changeInitialCharToUpperCase) {
    StringBuffer sBuff = new StringBuffer();
    char[] nameChars = name.toCharArray();
    for (int i=0; i<nameChars.length; i++) {
      if (nameChars[i]=='_' && i<(nameChars.length-1)) {
        sBuff.append(Character.toUpperCase(nameChars[++i]));
      } else if (changeInitialCharToUpperCase && i==0){
        sBuff.append(Character.toUpperCase(nameChars[i]));
      } else {  
        sBuff.append(Character.toLowerCase(nameChars[i]));
      }
    }
    return sBuff.toString();
  }
  
  /**
   * リモートデータベースに接続するダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   */
  public <E extends Serializable> E $useRemoteDummy(Class<? extends E> type, String remoteId) {
    E entity = $useDummy(type, null);
    $useRemote((ITableDef) entity, remoteId);
    return entity;
  }
  
  public void $useRemote(ITableDef entity, String remoteId) {
    if (idToRemote==null) {
      idToRemote = new HashMap<String, Remote>();
    }
    if (!idToRemote.containsKey(remoteId)) {
      idToRemote.put(remoteId, new Remote(remoteId));
    }
    Remote remote = idToRemote.get(remoteId);
    if (!remote.entiies.contains(entity)) {
      remote.entiies.add((ITableDef) entity);
      if (entityToRemote==null) {
        entityToRemote = new HashMap<ITableDef, Remote>();
      }
      entityToRemote.put((ITableDef) entity, remote);
    }
  }
  
  /**
   * ダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   */
  public <E extends Serializable> E $useDummy(Class<? extends E> type) {
    return $useDummy(type, null);
  }
  /**
   * ダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   */
  public <E extends Serializable> E $useDummy(Class<? extends E> type, String alias) {
    E entity;
    try {
      entity = (E) type.newInstance();
    } catch (Exception e1) {
      throw new RuntimeException(e1);
    }
    _useDummy(entity, alias);
    return (E)entity;
  }
  
  private <E extends Serializable> void _useDummy(E entity, String alias){
    ITableDef.MetaData<E> meta;
    if (ITableDef.class.isAssignableFrom(entity.getClass())) {
      try {
        meta = (MetaData<E>)((ITableDef)entity).createMetaData();
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    } else {
      meta = new ITableDef.MetaData<E>(entity.getClass());
    }
    if (alias!=null) meta.setAlias(alias);
    meta.fillDummyValue(entity);
  }
  
  /**
   * SELECTオブジェクトを作成します。.<br/>
   * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
   * 省略すると[SELECT * FROM ...] と同じ意味になります。
   * @param columns ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
   */
  public Select select(Serializable...columns) {
    Select select = new Select();
    if (query == null) {
      if ($$!=null) {
        select.into($$);
      }
      setQuery(select);
    }
    select.values(columns);
    return select;
  }
  
  /**
   * UPDATEオブジェクトを作成します。.<br/>
   * @param entity 更新対象テーブルのダミーエンティティオブジェクト
   * @return Update オブジェクト
   */
  public Update update(Serializable entity) {
    Update update = new Update(entity);
    setQuery(update);
    return update;
  }

  /**
   * UPDATEオブジェクトを作成します。.<br/>
   * SQL&lt;T&gt;で指定したエンティティクラスTが更新対象テーブルとなります.<br/>
   * @return Update オブジェクト
   */
  public Update update() {
    Update update = new Update($$);
    setQuery(update);
    return update;
  }
  
  /**
   * DELETEオブジェクトを作成します。.<br/>
   * @param entity 削除対象テーブルのダミーエンティティオブジェクト
   * @return Update オブジェクト
   */
  public Delete delete(Serializable entity) {
    Delete delete = new Delete(entity);
    setQuery(delete);
    return delete;
  }

  /**
   * DELETEオブジェクトを作成します。.<br/>
   * SQL&lt;T&gt;で指定したエンティティクラスTが削除対象テーブルとなります.<br/>
   * @return Update オブジェクト
   */
  public Delete delete() {
    Delete delete = new Delete($$);
    setQuery(delete);
    return delete;
  }
  
  /**
   * UPDATEオブジェクトを作成します。.<br/>
   * @param entity 更新対象テーブルのダミーエンティティオブジェクト
   * @return Update オブジェクト
   */
  public Insert insert(Serializable entity) {
    Insert insert = new Insert(entity);
    setQuery(insert);
    return insert;
  }

  /**
   * UPDATEオブジェクトを作成します。.<br/>
   * SQL&lt;T&gt;で指定したエンティティクラスTが更新対象テーブルとなります.<br/>
   * @return Update オブジェクト
   */
  public Insert insert() {
    Insert insert = new Insert($$);
    setQuery(insert);
    return insert;
  }
  /**
   * バインドパラメータを作成します.<br/>
   * @param パラメータ名
   */
  public BindParameter $bindParam(String name) {
    return new BindParameter(name);
  }
  
  /**
   * バインドパラメータクラス
   */
  static public class BindParameter implements Serializable {
    String name;
    public BindParameter(String name){
      this.name = name;
    }
  }
  
  /**
   * 条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  public Condition $if(Condition nestedCondition) {
    Condition rootCondition = new Condition(null, null, nestedCondition);
    Condition nextCondition = new Condition(null, rootCondition);
    return nextCondition;
  }

  
  /**
   * 条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  public Comparer $if(Serializable value) {
    return new Condition(null, null).$if(value);
  }
  
  /**
   * NOT( nestedCondition )条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  public Condition $not(Condition nestedCondition) {
    Condition rootCondition = new Condition(null, null, nestedCondition);
    rootCondition.conjunction = "NOT";
    rootCondition.negative = true;
    Condition nextCondition = new Condition(null, rootCondition);
    return nextCondition;
  }
  /**
   * NOT ( A ? B ) 条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  public Comparer $not(Serializable value) {
    return new Condition(null, null).$not(value);
  }
  /**
   * キーを指定してダイナミック条件式を設定します.<br/>
   * @param key 条件式を識別するキー
   */
  public void setDynamicCondtion(String key, Condition condition) {
    if (query!=null)
      query.getDynamicCondtions().get(key).setNestedCondition(condition);
  }
  
  /**
   * 設定済のダイナミック条件式を全てクリアします。
   */
  public void resetDynamicCondtions() {
    if (query!=null) query.resetDynamicCondtions();
  }
  /**
   * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  public Condition $evaluateNonNulls(Condition nestedCondition) {
    Condition rootCondition = new Condition(null, null, nestedCondition);
    Condition nextCondition = new Condition(null, rootCondition);
    rootCondition.setEvaluateNonNulls(true);
    return nextCondition;
  }  
  
  /**
   * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  public Comparer $evaluateNonNulls(Serializable value) {
    Condition condition = new Condition(null, null);
    condition.setEvaluateNonNulls(true);
    return condition.$if(value);
  }

  /**
   * SQL式を開始します.<br/>
   * @param source ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
   */
  public Expression $(Serializable source) {
    return new Expression(null, null, source);
  }
 
  static String space(int size) {
    char[] c = new char[size];
    for (int i=0; i < size; i++) {
      c[i] = ' ';
    }
    return new String(c);
  }
  
  /**
   * 
   * @author littlesoft
   *
   */
  public static class Token implements Serializable {
    protected Query query;
    private Token previous = null;
    private Token next = null;
    protected Token(Query query, Token previous) {
      setPrevious(previous);
      setQuery(query);
    }
    
    void setQuery(Query query) {
      this.query = query;
      if (next!=null) next.setQuery(query);
    }
    
    Class getType() { return null; };
    
    protected Query getQuery() {
      return query;
    }
    
    protected boolean isParameter(Serializable value) {
      if (value == null) return false;
      if (value instanceof BindParameter) return true;
      if (value instanceof String && ((String)value).charAt(0)==':') {
        return true;
      }
      if (value.getClass().isArray()) {
        for (Serializable v : (Serializable[])value) {
          if (!isParameter(v)) return false; 
        }
        return true;
      } 
      if (getQuery().isUsingField(value)) {
        return false;
      }
      return true;
    }
    
    protected void writeSQL(StringBuilder builder) {
      builder.append(this.toString());
      if (getNext()!=null) getNext().writeSQL(builder);
    }

    protected Token getPrevious() {
      return previous;
    }
    
    protected void setPrevious(Token previous) {
      if (previous == this.previous) return;
      if (this.previous != null) this.previous.next = null;
      this.previous = previous;;
      if (this.previous != null) this.previous.next = this;
    }
    
    protected Token getNext() {
      return next;
    }

    protected void setNext(Token next) {
      this.next = next;
    }

    protected Token getFirst() {
      Token t = this;
      while(t.getPrevious()!=null) {
        t = t.getPrevious();
      }
      return t;
    }
    
    protected Token getLast() {
      Token t = this;
      while(t.getNext()!=null) {
        t = t.getNext();
      }
      return t;
    }
  }
  
  /**
   * SQL内の式を表現するクラス.<br/>
   * @author littlesoft
   */
  public static class Expression extends Token {
    enum Type {
      DEFAULT, ALIAS, ACCEPT,
      ASCENDING, DESCENDING, NULLSFIRST, NULLSLAST,
      JOIN, INTO, 
      AVG, COUNT, MAX, MIN, SUM,
      ADD, DIVIDE, MULITIPLY, SUBTRACT
    };
    Type expressionType = Type.DEFAULT;
    Serializable source;
    Serializable dest;

    
    public Expression(Query query, Token previous, Serializable source) {
      super(query, previous);
      this.source = checkExpression(source);
    }
    
    @Override
    Class getType() {
      Class result = null;
      //sourceから型を推測
      if (source!=null) {
        if (source instanceof Expression) {
          result = ((Expression)source).getType();
        } else if (getQuery().getRoot().isUsingField(source) || !isParameter(source)) {
          result =  source.getClass();
        }
      //destから型を推測
      }
      if (result!=null) return result;
      if (dest!=null) {
        if (dest instanceof Expression) {
          result = ((Expression)dest).getType();
        } else if (getQuery().getRoot().isUsingField(dest) || !isParameter(dest)) {
          result =  dest.getClass();
        } 
      }
      return result;
    }    
    
    /**
     * この式の結果にエリアスを割り当てます。
     * @param alias エリアス名
     * @return Expression
     */
    public Expression alias(String alias) {
      this.expressionType = Type.INTO;
      this.dest = alias;
      return this;
    }

    /**
     * この式の結果をdestへ送ります.<br/>
     * destはダミーエンティティオブジェクトまたはそのフィールドを指定します。
     * @param dest
     * @return Expression
     */
    public Expression into(Serializable dest) {
      this.expressionType = Type.INTO;
      this.dest = dest;
      return this;
    }

    /**
     * 式[ this = expression]を作成します.<br/>
     * expressionはダミーエンティティオブジェクトのフィールド、またはExpressionオブジェクトを指定します。
     * @param dest
     * @return Expression
     */
    public Expression accept(Serializable expression) {
      this.expressionType = Type.ACCEPT;
      this.dest = checkExpression(expression);
      return this;
    }

    /**
     * ORDER BYの項目に昇順[ASC]を指定します.<br/>
     * @return Expression
     */
    public Expression ascending() {
      this.expressionType = Type.ASCENDING;
      this.dest = null;
      return new Expression(getQuery(), this, null);
    }

    /**
     * ORDER BYの項目に降順[DESC]を指定します.<br/>
     * @return Expression
     */
    public Expression descending() {
      this.expressionType = Type.DESCENDING;
      this.dest = null;
      return new Expression(getQuery(), this, null);
    }

    /**
     * ORDER BYの項目に[NULLSFIRST]を指定します.<br/>
     * @return Expression
     */
    public Expression nullsFirst() {
      this.expressionType = Type.NULLSFIRST;
      this.dest = null;
      return this;
    }

    /**
     * ORDER BYの項目に[NULLSLAST]を指定します.<br/>
     * @return Expression
     */
    public Expression nullsLast() {
      this.expressionType = Type.NULLSLAST;
      this.dest = null;
      return this;
    }
    
    /**
     * SQL式[ AVG(Expression) ]を生成します。.<br/>
     * @return Expression
     */
    public Expression avg() {
      this.expressionType = Type.AVG;
      this.dest = null;
      return new Expression(getQuery(), this, this);
    }

    /**
     * SQL式[ COUNT(Expression) ]を生成します。.<br/>
     * @return Expression
     */
    public Expression count() {
      this.expressionType = Type.COUNT;
      this.dest = null;
      return new Expression(getQuery(), this, this);
    }
    
    /**
     * SQL式[ MAX(Expression) ]を生成します。.<br/>
     * @return Expression
     */
    public Expression max() {
      this.expressionType = Type.MAX;
      this.dest = null;
      return new Expression(getQuery(), this, this);
    }
    
    /**
     * SQL式[ MIN(Expression) ]を生成します。.<br/>
     * @return Expression
     */
    public Expression min() {
      this.expressionType = Type.MIN;
      this.dest = null;
      return new Expression(getQuery(), this, this);
    }
    
    /**
     * SQL式[ SUM(Expression) ]を生成します。.<br/>
     * @return Expression
     */
    public Expression sum() {
      this.expressionType = Type.SUM;
      this.dest = null;
      return new Expression(getQuery(), this, this);
    }
    
    /**
     * SQL式[ Expression + value ]を生成します。.<br/>
     * @param value 
     * @return Expression
     */
    public Expression add(Serializable value) {
      this.expressionType = Type.ADD;
      this.dest = checkExpression(value);
      return new Expression(getQuery(), this, this);
    }

    /**
     * SQL式[ Expression / value ]を生成します。.<br/>
     * @param value
     * @return Expression
     */
    public Expression divide(Serializable value) {
      this.expressionType = Type.DIVIDE;
      this.dest = checkExpression(value);
      return new Expression(getQuery(), this, this);
    }

    /**
     * SQL式[ Expression * value ]を生成します。.<br/>
     * @param value
     * @return Expression
     */
    public Expression multiply(Serializable value) {
      this.expressionType = Type.MULITIPLY;
      this.dest = checkExpression(value);
      return new Expression(getQuery(), this, this);
    }
    
    /**
     * SQL式[ Expression - value ]を生成します。.<br/>
     * @param value
     * @return Expression
     */
    public Expression subtract(Serializable value) {
      this.expressionType = Type.SUBTRACT;
      this.dest = checkExpression(value);
      return new Expression(getQuery(), this, this);
    }

    /**
     * SQL式[ INNER JOIN table ]を生成します.<br>
     * @param table
     * @return Join
     */
    public Join innerJoin(Serializable table) {
      return addJoin("INNER", table);
    }
    
    /**
     * SQL式[ LEFT OUTER JOIN table ]を生成します.<br>
     * @param table
     * @return Join
     */
    public Join leftOuterJoin(Serializable table) {
      return addJoin("LEFT OUTER", table);
    }

    /**
     * SQL式[ RIGHT OUTER JOIN table ]を生成します.<br>
     * @param table
     * @return Join
     */
    public Join rightOuterJoin(Serializable table) {
      return addJoin("RIGHT OUTER", table);
    }
    
    private Join addJoin(String option, Serializable table) {
      this.expressionType = Type.JOIN;
      NestedJoin joinSelect = new NestedJoin(source);
      setQuery(joinSelect);
      Join j = joinSelect.addJoin(option, table, null);
      j.setPrevious(this);
      return j;
    }
    
    private Serializable checkExpression(Serializable obj) {
      if (obj instanceof Expression) {
        Expression ex = (Expression)((Expression)obj).getFirst();
        ex.setQuery(getQuery());
        return ex;
      }
      return obj;
    }
    
    String toString(Serializable obj) {
      if (obj==null) {
        return "";
      } else if (obj instanceof Select) {
        Select sel = (Select)obj;
        sel.setParent(getQuery());
        return "(" + sel.createSQL(getQuery().level+1) + ") ";
      } else if (obj instanceof Expression) {
        Expression exp = (Expression)obj;
        StringBuilder builder = new StringBuilder();
        builder.append("(");
        exp.setQuery(getQuery());
        exp.writeSQL(builder);
        builder.append(")");
        return builder.toString();
      } else if (getQuery().isUsingEntity(obj)) {
        return getQuery().getTableOrAliasName(obj);
      //} else if (getQuery().isUsingField(obj)) {
      //  return getQuery().getColumnOrParameterName(this, obj, false);
      } else {
        try {
          return getQuery().getColumnOrParameterName(this, obj, false);
        } catch (NullParameterNotifycation e) {
          return "";
        }
        //return obj.toString();
      }
    }
    
    @Override
    public String toString() {
      String result = "";
      switch(expressionType) {
      case DEFAULT:
        result = (getNext()==null) ? "" :  toString(source);
        break;
      case INTO:
        if (dest!=null && getQuery().isUsingField(dest)) {
          result = ((getFirst()!=this) ? "" : toString(source)) + " " + getQuery().getColumnName(dest);
        }
        break;
      case ACCEPT:
        result = (toString(source) + " = " + toString(dest));
        break;
      case ASCENDING:
        result = toString(source) + " ASC";
        break;
      case DESCENDING:
        result = toString(source) + " DESC";
        break;
      case AVG:
        result = "AVG(" + toString(source) + ")";
        break;
      case COUNT:
        result = "COUNT(" + ((source==null) ? "*" : toString(source)) + ")";
        break;
      case MAX:
        result = "MAX(" + toString(source) + ")";
        break;
      case MIN:
        result = "MIN(" + toString(source) + ")";
        break;
      case SUM:
        result = "SUM(" + toString(source) + ")";
        break;
      case ADD:
        if (String.class.equals(getType())){
          result = ((getFirst()!=this) ? "" : toString(source)) + " || " + toString(dest);
        } else {
          result = ((getFirst()!=this) ? "" : toString(source)) + " + " + toString(dest);
        }
        break;   
      case DIVIDE:
        result = ((getFirst()!=this) ? "" : toString(source)) + " / " + toString(dest);
        break;   
      case SUBTRACT:
        result = ((getFirst()!=this) ? "" : toString(source)) + " - " + toString(dest);
        break;   
      case MULITIPLY:
        result = ((getFirst()!=this) ? "" : toString(source)) + " * " + toString(dest);
        break;   
      case NULLSFIRST:
        result = " NULLS FIRST";
        break;
      case NULLSLAST:
        result = " NULLS LAST";
        break;
      case JOIN:
        if (getQuery().isUsingEntity(source)) {
          result = getQuery().getTableAsAliasOrTemporaryName(source);
        } else {
          result = toString(source);
        }
      }
      return result;
    }
  }
  
  
  static class NullParameterNotifycation extends Exception {
    NullParameterNotifycation() {
      super("NullParameterNotifycation");
    }
  }
  /**
   * 条件式を表現するクラス.<br/>
   * @author littlesoft
   */
  public static class Condition<Q> extends Token {
    
    /**
     * where, $if, onで開始した条件式の終了を示す変数
     */
    public Q end;
    Condition nestedCondition = null;
    Serializable value = null;
    String conjunction = null;
    boolean withoutConjunction = false;
    boolean evaluateNonNulls = false;
    boolean negative = false; 

    Condition(Query query, Token previous) {
      super(query, previous);
    }
    
    Condition(Query query, Token previous, Condition condition) {
      super(query, previous);
      setNestedCondition(condition);
    }
    
    void setNestedCondition(Condition condition) {
      this.nestedCondition = (Condition)condition.getFirst(); 
      this.nestedCondition.setQuery(query);      
    }
    
    @Override
    Class getType() {
      if (value!=null) {
        return value.getClass();
      } else if (nestedCondition!=null) {
        return nestedCondition.getClass();
      } else {
        return Object.class;
      }
    }
    
    Comparer<Q,?> $if(Serializable value) {
      this.nestedCondition = null;
      this.value = value;
      conjunction = null;
      return (Comparer<Q, ?>) createComparer(getQuery(), this, Query.class, value.getClass());
    }
    
//    Comparer<Q,?> $if(Condition condition) {
//      this.nestedCondition = condition;
//      this.value = null;
//      conjunction = null;
//      return (Comparer<Q, ?>) createComparer(getQuery(), this, getQuery().getClass(), condition.getType());
//    }

    Comparer<Q,?> $not(Serializable value) {
      this.nestedCondition = null;
      this.value = value;
      conjunction = "NOT (";
      negative = true;
      return (Comparer<Q, ?>) createComparer(getQuery(), this, Query.class, value.getClass());
    }

//    Comparer<Q,?> $not(Condition condition) {
//      this.nestedCondition = condition;
//      this.value = null;
//      conjunction = "NOT";
//      return (Comparer<Q, ?>) createComparer(getQuery(), this, getQuery().getClass(), condition.getType());
//    }

    @Override
    void setQuery(Query query) {
      super.setQuery(query);
      if (nestedCondition!=null) {
        nestedCondition.setQuery(query);
      }
      //if (query instanceof Select) {
        this.end = (Q)query;
      //}
    }

    /**
     * 条件式[ this AND (condition) ]を生成します.<br/>
     * @param condition
     * @return Condition
     */
    public Condition<Q> and(Condition condition) {
      conjunction = "AND";
      nestedCondition = (Condition)condition.getFirst(); 
      nestedCondition.setQuery(query);
      value = null;
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this AND value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Comparer
     */
    public Comparer<Q,?> and(Serializable value) {
      conjunction = "AND";
      nestedCondition = null;
      this.value = value;
      return (Comparer<Q, ?>) createComparer(getQuery(), this, getQuery().getClass(), value.getClass());
    }

    /**
     * 条件式[ this AND NOT(condition) ]を生成します.<br/>
     * @param condition
     * @return Condition
     */
    public Condition<Q> andNot(Condition condition) {
      conjunction = "AND NOT";
      nestedCondition = (Condition)condition.getFirst(); 
      nestedCondition.setQuery(query);
      negative = true;
      value = null;
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this AND NOT ( value ... ) ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Comparer
     */
    public Comparer<Q,?> andNot(Serializable value) {
      conjunction = "AND NOT(";
      nestedCondition = null;
      negative = true;
      this.value = value;
      return (Comparer<Q, ?>) createComparer(getQuery(), this, getQuery().getClass(), value.getClass());
    }
    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであれば[ this AND (condition) ]を生成します.<br/>
     * @param condition
     * @return Condition
     */
    public Condition<Q> andEvaluateNonNulls(Condition condition) {
      conjunction = "AND";
      this.nestedCondition = (Condition)condition.getFirst(); 
      this.nestedCondition.setQuery(query);
      this.value = null;
      setEvaluateNonNulls(true);
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであれば[ this AND value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Comparer
     */
    public Comparer<Q,?> andEvaluateNonNulls(Serializable value) {
      conjunction = "AND";
      this.nestedCondition = null;
      this.value = value;
      setEvaluateNonNulls(true);
      return (Comparer<Q, ?>) createComparer(getQuery(), this, getQuery().getClass(), value.getClass());
    }

    
    /**
     * 実行の際、動的に設定される条件式のキーを設定します.<br/>
     * @param String 条件式キー
     * @return Condition
     */
    public Condition<Q> andDynamicCondition(String key) {
      conjunction = "AND";
      this.nestedCondition = new DynamicCondition<SQL.Select>(getQuery(), this, key);
      this.value = null;
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this OR (condition) ]を生成します.<br/>
     * @param condition
     * @return Condition
     */
    public Condition<Q> or(Condition condition) {
      conjunction = "OR";
      this.nestedCondition = (Condition)condition.getFirst(); 
      this.nestedCondition.setQuery(query);
      this.value = null;
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this OR NOT (condition) ]を生成します.<br/>
     * @param condition
     * @return Condition
     */
    public Condition<Q> orNot(Condition condition) {
      conjunction = "OR NOT";
      negative = true;
      nestedCondition = (Condition)condition.getFirst(); 
      nestedCondition.setQuery(query);
      value = null;
      return new Condition<Q>(getQuery(), this);
    }
    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであれば条件式[ this OR value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Comparer
     */
    public Comparer<Q,?> orEvaluateNonNulls(Serializable value) {
      conjunction = "OR";
      this.nestedCondition = null; 
      this.value = value;
      setEvaluateNonNulls(true);
      return (Comparer<Q, ?>) createComparer(getQuery(), this, Query.class, value.getClass());
    }
    
    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであれば条件式[ this OR (condition) ]を生成します.<br/>
     * @param condition
     * @return Condition
     */
    public Condition<Q> orEvaluateNonNulls(Condition condition) {
      conjunction = "OR";
      this.nestedCondition = (Condition)condition.getFirst(); 
      this.nestedCondition.setQuery(query);
      this.value = null;
      setEvaluateNonNulls(true);
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 実行の際、動的に設定される条件式のキーを設定します.<br/>
     * @param String 条件式キー
     * @return Condition
     */
    public Condition<Q> orDynamicCondition(String key) {
      conjunction = "OR";
      this.nestedCondition = new DynamicCondition<SQL.Select>(getQuery(), this, key);
      this.value = null;
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this OR value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Comparer
     */
    public Comparer<Q,?> or(Serializable value) {
      conjunction = "OR";
      this.nestedCondition = null; 
      this.value = value;
      return (Comparer<Q, ?>) createComparer(getQuery(), this, Query.class, value.getClass());
    }
    
    /**
     * 条件式[ this OR NOT ( value...) ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Comparer
     */
    public Comparer<Q,?> orNot(Serializable value) {
      conjunction = "OR NOT (";
      negative = true;
      nestedCondition = null; 
      this.value = value;
      return (Comparer<Q, ?>) createComparer(getQuery(), this, Query.class, value.getClass());
    }

    @Override
    public String toString() {
      if (!getQuery().includeNullParamCondition && isEvaluateNonNulls() && !getQuery().isUseAbsoluteFakeCondition()) {
        try {
          checkNullParams();
          if (getFirst()==this) {
            Token t = getNext();
            while(t!=null) {
              if (t instanceof Condition) {
                ((Condition)t).withoutConjunction = false;
                break;
              }
              t = t.getNext();
            }
          }
        } catch (NullParameterNotifycation x) {
          if (getFirst()==this) {
            Token t = getNext();
            while(t!=null) {
              if (t instanceof Condition) {
                ((Condition)t).withoutConjunction = true;
                break;
              }
              t = t.getNext();
            }
          }
          return "";
        }
      }
      String result;
      if (isNegative() && withoutConjunction) {
        if (nestedCondition == null) {
          result = " NOT (";
        } else {
          result = " NOT ";
        }
      } else {
        result = (withoutConjunction || conjunction==null) ? "" : " " + conjunction + " ";
      }
      try {
        if (nestedCondition!=null) {
          StringBuilder builder = new StringBuilder();
          //入れ子の条件
          if (nestedCondition instanceof DynamicCondition) {
            if (nestedCondition.nestedCondition == null) {
              return "";
            } else {
              nestedCondition.writeSQL(builder);
            }
          } else {
            builder.append("(");
            nestedCondition.writeSQL(builder);
            builder.append(")");
          }
          result += builder.toString();
        } else if (value!=null){
          result += getQuery().getColumnOrParameterName(this, value, isEvaluateNonNulls());
        } else {
          return "";
        }
      } catch (NullParameterNotifycation e) {
        return "";
      }
      return result;
    }

    void checkNullParams() throws NullParameterNotifycation {
      Condition condition = this;
      while (condition!=null) {
        if (condition.value!=null && !getQuery().isUsingField(condition.value)) {
          getQuery().getColumnOrParameterName(condition, condition.value, true);
        }
        if (condition.getNext() instanceof Comparer<?, ?>) {
          Comparer<?, ?> comparer = (Comparer<?, ?>)condition.getNext();
          comparer._toString();
        }
        if (condition.nestedCondition!=null) {
          condition = condition.nestedCondition;
        } else {
          condition = null;
        }
      }
    }
    
    
    boolean isEvaluateNonNulls() {
      return evaluateNonNulls;
    }

    void setEvaluateNonNulls(boolean evaluateNonNulls) {
      this.evaluateNonNulls = evaluateNonNulls;
      if (nestedCondition != null) {
        nestedCondition.setEvaluateNonNulls(evaluateNonNulls);
      }
    }

    @Override
    protected void writeSQL(StringBuilder builder) {
      builder.append(this.toString());
      if (getNext()!=null) getNext().writeSQL(builder);
    }

    public boolean isNegative() {
      return negative;
    }
    
    public boolean isExportable(Serializable remoteTable) {
      Comparer comp = null;
      if (getNext() instanceof Comparer) {
        comp = (Comparer)getNext();
      } else {
        return false;
      }
      if ( (getQuery().isRemote(remoteTable, value) && isParameter(comp.values)) ||
           (isParameter(value) && getQuery().isRemote(remoteTable, comp.values))){
        return true;
      }
      return false;
    }
    
  }
  
  static public <Q, T> Comparer<Q,T> createComparer(Query query, Token previous, 
      Class<? extends Q> quertType, Class<? extends T> dataType){ 
    return new Comparer<Q, T>(query, previous);
  }

  /**
   * 
   * @author littlesoft
   *
   * @param <Q>
   */
  public static class DynamicCondition<Q> extends Condition<Q> {

    private String name = null;
    
    DynamicCondition(Query query, Token previous, String name) {
      super(query, previous);
      this.name = name;
      if (query!=null) {
        query.getDynamicCondtions().put(name, this);
      }      
    }

    @Override
    void setQuery(Query query) {
      super.setQuery(query);
      if (query!=null && this.name!=null) {
        query.getDynamicCondtions().put(name, this);
      }
    }
    
    @Override
    public String toString() {
      if (nestedCondition!=null) {
        //動的条件の挿入
        String result = (withoutConjunction || conjunction==null) ? "" : " " + conjunction + " ";
        StringBuilder builder = new StringBuilder();
        nestedCondition.writeSQL(builder);
        result += builder.toString();
        return result;
      } else {
        return "";
      }
    }
  }
  
  /**
   * 比較演算子を表現するクラス.<br/>
   * @author littlesoft
   */
  public static class Comparer<Q,T> extends Token {

    String comparer = null;
    Serializable[] values = null;
    enum Type {DEFAULT,BETWEEN,BIGGER,BIGGER_EQUAL,IN,IS,IS_NULL,IS_NOT_NULL,LIKE,SMALLER,SMALLER_EQUAL, START_WITH, END_WITH, CONTAINS}
    Type type = Type.DEFAULT;
    
    public Comparer(Query query, Token previous) {
      super(query, previous);
    }
    
    @Override
    Class getType() {
      if (getPrevious()!=null && getPrevious() instanceof Condition) {
        return ((Condition)getPrevious()).getType();
      } else {
        return String.class;
      }
    }
    /**
     * 条件式[ this = value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */
    public Condition<Q> is(Serializable value) {
      comparer = " = ";
      setValue(value);
      return new Condition<Q>(getQuery(), this);
    }
    
    private void setValue(Serializable value) {
      if (value.getClass().isArray()) {
        this.values = (Serializable[])value;
      } else {
        this.values = new Serializable[]{value};
      }
    }
    
    /**
     * 条件式[ this <> value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */
    public Condition<Q> isNot(Serializable value) {
      comparer = " <> ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this > value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */
    public Condition<Q> bigger(Serializable value) {
      comparer = " > ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this >= value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */
    public Condition<Q> biggerEqual(Serializable value) {
      comparer = " >= ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this < value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */
    public Condition<Q> smaller(Serializable value) {
      comparer = " < ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this <= value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */
    public Condition<Q> smallerEqual(Serializable value) {
      comparer = " <= ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this IS NULL ]を生成します.<br/>
     * @return Condition
     */
    public Condition<Q> isNull() {
      comparer = " IS NULL";
      this.values = new Serializable[0];
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this IS NOT NULL ]を生成します.<br/>
     * @return Condition
     */
    public Condition<Q> isNotNull() {
      comparer = " IS NOT NULL";
      this.values = new Serializable[0];
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this LIKE value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */    
    public Condition<Q> like(Serializable value) {
      comparer = " LIKE ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this LIKE value || '%' ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */    
    public Condition<Q> startsWith(Serializable value) {
      type = Type.START_WITH;
      comparer = " LIKE ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this LIKE '%' || value ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */    
    public Condition<Q> endsWith(Serializable value) {
      type = Type.END_WITH;
      comparer = " LIKE ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this LIKE '%' || value || '%' ]を生成します.<br/>
     * @param value ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */    
    public Condition<Q> contains(Serializable value) {
      type = Type.CONTAINS;
      comparer = " LIKE ";
      this.values = new Serializable[]{value};
      return new Condition<Q>(getQuery(), this);
    }
    
    /**
     * 条件式[ this BETWEEN value1 AND value2 ]を生成します.<br/>
     * @param value1,value2 ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */    
    public Condition<Q> between(Serializable value1, Serializable value2) {
      type = Type.BETWEEN;
      comparer = " BETWEEN ";
      this.values = new Serializable[]{value1, value2};
      return new Condition<Q>(getQuery(), this);
    }

    /**
     * 条件式[ this IN (value1, value2, ...) ]を生成します.<br/>
     * @param values 左記の配列：ダミーエンティティオブジェクトのフィールド, 固定値, BindParamオブジェクト, Expressionオブジェクト
     * @return Condition
     */    
    public Condition<Q> in(Serializable... values) {
      type = Type.IN;
      comparer = " IN ";
      this.values = values;
      return new Condition<Q>(getQuery(), this);
    }
    
    
    
    @Override
    public String toString() {
      try {
        return _toString();
      } catch (NullParameterNotifycation e) {
        return "";
      }
    }
    
    String _toString() throws NullParameterNotifycation {
      Condition prevToken = ((Condition) getPrevious());
      boolean isNonNullsEvaluation = prevToken.isEvaluateNonNulls();
      String end = (prevToken.isNegative() &&  prevToken.nestedCondition==null) ?  ")" : "";
      switch (type) {
      case BETWEEN:
        String from = null;
        String to = null;
        try {
          from = getQuery().getColumnOrParameterName(this, values[0], isNonNullsEvaluation);
        } catch (NullParameterNotifycation n1) {
        }
        try {
          to = getQuery().getColumnOrParameterName(this, values[1], isNonNullsEvaluation);
        } catch (NullParameterNotifycation n1) {
        }
        if (from != null && to != null) {
          return comparer + from + " AND " + to + end;
        } else if (from != null) {
          return " >= " + from + end;
        } else if (to != null) {
          return " <= " + to + end;
        } else {
          throw new NullParameterNotifycation();
        }
      case IN:
        StringBuilder s = new StringBuilder();
        s.append(comparer);
        s.append("(");
        String c;
        int count = 0;
        for (Serializable v : values) {
          c = (count > 0) ? "," : "";
          try {
            c += getQuery().getColumnOrParameterName(this, v, isNonNullsEvaluation);
            s.append(c);
            count++;
          } catch (NullParameterNotifycation n1) {}
        }
        if (count == 0) throw new NullParameterNotifycation();
        s.append(")");
        return s.toString() + end;
      case START_WITH:
        return comparer + simpleValue(values[0], isNonNullsEvaluation) + " || '%' " + end;
      case END_WITH:
        return comparer + "'%' || " + simpleValue(values[0], isNonNullsEvaluation) + end;
      case CONTAINS:
        return comparer + "'%' || " + simpleValue(values[0], isNonNullsEvaluation) + " || '%' " + end;
      default:
        if (values!=null && values.length>0)
          return comparer + simpleValue(values[0], isNonNullsEvaluation) + end;
        else
          return comparer + end;
      }
    }
    
    private String simpleValue(Serializable value, boolean isNonNullsEvaluation) throws NullParameterNotifycation {
      return (values.length == 0) ? "" : getQuery().getColumnOrParameterName(this, values[0], isNonNullsEvaluation);
    }
  }
  /**
   * SQLクエリー基底クラス
   * @author littlesoft
   */
  public static abstract class Query extends Token implements Serializable {
    SQL sqlContext;
    Query parent;
    int level = 0;
    IdentityHashMap<Serializable, ITableDef.MetaData> entityToMeta = new IdentityHashMap<Serializable, ITableDef.MetaData>();
    IdentityHashMap<Serializable, Column>    entityFieldToColumn = new IdentityHashMap<Serializable, Column>();
    IdentityHashMap<Serializable, ITableDef.MetaData> entityFieldToMeta = new IdentityHashMap<Serializable, ITableDef.MetaData>();
    ArrayList<String> nameList = new ArrayList<String>();
    boolean includeNullParamCondition = false;
    boolean prepared = false;
    HashMap<String, DynamicCondition> dynamicCondtions = null;
    boolean onEditingRemoteSelectSQL = false;
    boolean useAbsoluteFakeCondition = false;
    
    Query() {
      super(null, null);
    }
    

    public boolean isUseAbsoluteFakeCondition() {
      return useAbsoluteFakeCondition;
    }

    public void setUseAbsoluteFakeCondition(boolean useAbsoluteFakeCondition) {
      this.useAbsoluteFakeCondition = useAbsoluteFakeCondition;
      setPrepared(false);
    }

    HashMap<String, DynamicCondition> getDynamicCondtions() {
      if (dynamicCondtions == null) {
        dynamicCondtions = new HashMap<String, DynamicCondition>();
      }
      return dynamicCondtions;
    }
    
    void setDynamicCondtions(String key, DynamicCondition condition) {
      getDynamicCondtions().get(key).setNestedCondition(condition);
      setPrepared(false);
    }
    
    void resetDynamicCondtions() {
      for (DynamicCondition cond : dynamicCondtions.values()) {
        cond.nestedCondition = null;
      }
      setPrepared(false);
    }
    
    
    void setSQLContext(SQL sqlContext) {
      getRoot().sqlContext = sqlContext;
    }
    
    SQL getSQLContext() {
      return getRoot().sqlContext;
    }

    void setParent(Query parent) {      
      this.parent = parent;
      Query root = getRoot();
      if (root==this) return;
      for (Serializable key : entityToMeta.keySet()) {
        if (isEntityObject(key) && !isUsingEntity(key)) {
          useDummy(key);
        }
      }
      for (String name : getDynamicCondtions().keySet()) {
        root.getDynamicCondtions().put(name, dynamicCondtions.get(name));
      }
    }
    
    Query getParent() {
      return parent;
    }
    
    Query getRoot() {
      Query result = this;
      while (true) {
        if (result.getParent()==null) {
          break;
        } else {
          result = result.getParent();
        }
      }
      return result;
    }
    
    boolean isEntityObject(Serializable object) {
      if (object == null) return false;
      if (object.getClass().isPrimitive()) return false;
      if (object instanceof String || object instanceof Number || 
          object instanceof Query  || object instanceof Token) return false;
      if (getRoot().entityFieldToColumn.containsKey(object)) return false;
      return true;
    }
    
    
//    String getTableAsAlias(Serializable entity) {
//      return getRoot().entityToMeta.get(entity).getTableAsAlias();
//    }

    String getTableOrAliasName(Serializable entity) {
      MetaData meta = getRoot().entityToMeta.get(entity);
      if (isRemote(meta.$$) && !onEditingRemoteSelectSQL){
        //テンポラリーテーブルへあるいはそのエリアス名を蚊r巣
        return toTemporaryTableName(meta) ;
      } else {
        return meta.getTableOrAliasName();
      }
//      if (meta.hasAlias()) {
//        return meta.getTableOrAliasName();
//      } else {
//        return getTableAsAliasOrTemporaryName(meta);
//      }
    }

    String getAlias(Serializable entity) {
      return getRoot().entityToMeta.get(entity).getAlias();
    }
    
    String getDBTableName(Serializable entity) {
      return getRoot().entityToMeta.get(entity).getDBTableName(true);
    }
    
    public Serializable getDummy(String dbTableName) {
      for (Serializable entity : entityToMeta.keySet()) {
        ITableDef.MetaData meta= entityToMeta.get(entity);
        if (dbTableName.equals(meta.getDBTableName(false))) {
          return entity;
        }
      }
      return null;
    }

    public MetaData getMetaData(Serializable entity) {
      if (getRoot().entityToMeta.containsKey(entity)) {
        return getRoot().entityToMeta.get(entity);
      } else {
        return null;
      }
    }
    
    boolean isUsingEntity(Serializable entity) {
      return getRoot().entityToMeta.containsKey(entity);
    }
    
    boolean isUsingField(Serializable field) {
      return getRoot().entityFieldToColumn.containsKey(field);
    }

    public Params getParams() {
      return getSQLContext().getParams();
    }
    
    Serializable fieldToEntity(Serializable value) {
      return getRoot().entityFieldToMeta.get(value).$$;
    }
    
    boolean isRemote(Serializable object) {
      if (object.getClass().isArray()) {
        for (Serializable o : (Serializable[])object){
          if(!isRemote(o)) return false; 
        }
        return true;
      }
      return (getRoot().entityToMeta.containsKey(object) 
              && sqlContext != null
              && sqlContext.entityToRemote != null
              && sqlContext.entityToRemote.containsKey(object));
    }
    
    boolean isRemote(Serializable entity, Serializable value) {
      if (entity == value) {
        return isRemote(entity);
      }
      if (value.getClass().isArray()) {
        for (Serializable o : (Serializable[])value){
          if(!isRemote(entity, o)) return false; 
        }
        return true;
      }
      return  (getRoot().entityToMeta.containsKey(entity) 
               && getRoot().fieldToEntity(value) == entity
               && sqlContext.entityToRemote != null
               && sqlContext.entityToRemote.containsKey(entity));
    }
    
    
    public Query param(String name, Serializable value) {
      try {
        if (getParams().indexOfColumn(name)==-1) {
          getParams().addColumn(name, Column.classToColtypeMap(value.getClass()));
        }
        getParams().setValue(name, value);
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
      return this;
    }
    
    protected void writeList(List<String> list, StringBuilder builder) {
      int i=0;
      for (String s : list) {
        builder.append(indent() + ((i==0) ? " " : ",") + s + "\n");
        i++;
      }
    }
    
    @Override
    public String toString() {
      return this.createSQL();
    }
    
    public void systemOut() {
      System.out.println(this.createSQL());
    }
    
    <E extends Serializable> E useDummy(E entity){
      return useDummy(entity, null);    
    }

    <E extends Serializable> E useDummy(E entity, String alias){ 
      return setupDummy(entity, alias);
    }
    
    private <E extends Serializable> E setupDummy(E entity, String alias){ 
      try {
        
        ITableDef.MetaData<E> meta;
        if (entity instanceof ITableDef) {
          meta = (MetaData<E>) ((ITableDef)entity).createMetaData();
        } else {
          meta = new ITableDef.MetaData<E>(entity);
        }
        meta.fillDummyValue(entity);
        if (alias==null || "".equals(alias)) {
          String tableName = meta.getDBTableName(false);
          alias = tableName;
        }  
        String aname = alias;
        int i = 1;
        while(getRoot().nameList.contains(aname)) {
          aname = alias + String.valueOf(i++);
        }
        if (!aname.equals(alias)) meta.setAlias(aname);
        getRoot().nameList.add(aname);
        getRoot().entityToMeta.put(entity, meta);
        Serializable dummy = null;
        for (Column col : meta.getColumns()) {
          dummy = col.getEntityValue(entity);
          getRoot().entityFieldToColumn.put(dummy, col);
          getRoot().entityFieldToMeta.put(dummy, meta);
        }
        return entity;
      } catch (Exception x) {
        throw new RuntimeException(x);
      }
    }

    private void addParameter(String name, Class type) {
      Column.Type colType = Column.classToColtypeMap(type);
      addParameter(name, colType); 
    }
    
    private void addParameter(String name, Column.Type colType) {
      try {
        if (getParams().indexOfColumn(name)==-1) {
          getParams().addColumn(name, colType);
        } 
      } catch (DataException e) {
        throw new RuntimeException(e);
      }
    }

    protected boolean isParameter(Serializable value) {
      return ( value != null && (
                 (value instanceof BindParameter) ||
                 (value instanceof String && ((String)value).startsWith(":")) ||
                 (!isUsingField(value) && !isUsingEntity(value))
                 )
              );
    }
    
    public String getColumnOrParameterName(Token token, Serializable value, boolean nullParamNotify) throws NullParameterNotifycation {
      try {
        if (value!=null && isUsingField(value)){
          //テーブルカラム
          MetaData meta = getRoot().entityFieldToMeta.get(value);
          return ( (!onEditingRemoteSelectSQL && isRemote(meta.$$)) ?  toTemporaryTableName(meta) : getTableOrAliasName(meta.$$) )
              + "." + getRoot().entityFieldToColumn.get(value).getDBColumnName();
        } else if (value instanceof BindParameter) {
          //BindParameterオブジェクト
          BindParameter p = (BindParameter)value;
          if (p.name.length()==0 || Character.isDigit(p.name.charAt(0))) {
            throw new RuntimeException("invalid parameter name[" + p.name + "], first character must not be a digit.");
          }
          if (!includeNullParamCondition && nullParamNotify && (getParams().indexOfColumn(p.name)==-1 || getParams().isNull(p.name))) {
            throw new NullParameterNotifycation();
          }
          addParameter(p.name, token.getType());
          return ":" + p.name;
        } else if (value instanceof String && ((String)value).startsWith(":")) {
          //コロンで始まるパラメータ指定
          String pname = ((String)value).substring(1);
          if (!includeNullParamCondition && nullParamNotify && (getParams().indexOfColumn(pname)==-1 || getParams().isNull(pname) )) {
            throw new NullParameterNotifycation();
          }
          addParameter(pname, token.getType());
          return (String)value;
        } else if (value!=null) {
          //エンティティに存在しないので、パラメータと解釈する。
          if (token instanceof Condition<?> || token instanceof Comparer<?,?>) {
            String name = String.valueOf(token.hashCode());
            if (getParams().indexOfColumn(name) < 0) addParameter(name, token.getType());
            getParams().setValue(name, Column.lenientConvertNumber(value, token.getType()));
            return ":" + name;
          } else if (value instanceof String) {
            return quote(value.toString());
          } else if (value instanceof java.util.Date) {
            return quote(((java.util.Date) value).toString());
          } else if (value instanceof Number) {
            return value.toString();
          } else {
            return quote(LSJson.serialize(value, false));
          }
        } else {
          if (!includeNullParamCondition && nullParamNotify) throw new NullParameterNotifycation();
          return "null";
        }
      } catch (NullParameterNotifycation n) {
        throw n;
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    }
    
    private String quote(String value) {
      return "'" + value.replaceAll("\\'", "''") + "'" ;
    }
    
    public Column getColumn(Serializable value) {
      return getRoot().entityFieldToColumn.get(value);
    }
    
    public String getColumnName(Serializable value) {
      if (value!=null && isUsingField(value)){
        return getRoot().entityFieldToColumn.get(value).getDBColumnName();
      } else {
        throw new RuntimeException("invalid value : " + value);
      }
    }
    
    
    String createSQL(int level) {
      this.level = level;
      return createSQL();
    }
    
    String indent() {
      return SQL.space(level*2);
    }
    
    private String sql;
    
    /**
     * 
     * @param includeNullParamCondition
     * @return
     */
    public String getSQL(boolean includeNullParamCondition) {
      if (!isPrepared() || !includeNullParamCondition)
        _prepare(includeNullParamCondition);
      return sql;
    }
    
    private boolean isJSONObject(Column col) {
      return col.getType()!=null && col.getType().equals(Column.Type.JSONOBJECT);
    }
    
    void _prepare(boolean includeNullParamCondition) {
      try {
        HashMap<String, Serializable> temp1 = new HashMap<String, Serializable>();
        HashMap<String, Serializable> temp2 = new HashMap<String, Serializable>();
        for (String name : getParams().names()) {
          if (getParams().getColumn(name).isReadOnly()) continue;
          if (Character.isDigit(name.charAt(0)) || getParams().getValue(name)==null) continue;
          if (isJSONObject(getParams().getColumn(name))) {
            temp1.put(name, getParams().getValue(name));
          } else {
            temp2.put(name, getParams().getValue(name));
          }
        }
        getParams().clearColumns();
        if (!includeNullParamCondition) {
          //パラメータがNULLの条件はSQLから取り除かれる為、createSQLする前に既存の値をセットしておく
          for (String name : temp1.keySet()) {
            getParams().setValue(name, temp1.get(name));
          }
          for (String name : temp2.keySet()) {
            getParams().setValue(name, temp2.get(name));
          }
        }
        this.includeNullParamCondition = includeNullParamCondition;
        sql = createSQL();
        for (String name : getParams().names()) {
          if (getParams().getColumn(name).isReadOnly()) continue;
          if (temp1.containsKey(name)) {
            getParams().setValue(name, temp1.get(name));
          }
        }  
        for (String name : getParams().names()) {
          if (getParams().getColumn(name).isReadOnly()) continue;
          if (temp2.containsKey(name)) {
            getParams().setValue(name, temp2.get(name));
          }
        }
        prepared = true;
      } catch (Exception x) {
        throw new RuntimeException(x);
      } finally {
        this.includeNullParamCondition = false;
      }
    }
    
    abstract public String createSQL();
    
    abstract int execute(Connection conn, boolean pendingCommit, boolean reuseStatement) throws DataException;


    public boolean isPrepared() {
      return prepared;
    }

    public void setPrepared(boolean prepared) {
      this.prepared = prepared;
    }
    
    /**
     * テーブルASエリアス、またはテンポラリーテーブル名
     * @param dummy entity
     * @return テーブルASエリアス、またはテンポラリーテーブル名
     */
    String getTableAsAliasOrTemporaryName(Serializable dummy) {
      ITableDef.MetaData<?> meta = getRoot().entityToMeta.get(dummy);
      return getTableAsAliasOrTemporaryName(meta);
    }
    
    /**
     * テーブルASエリアス、またはテンポラリーテーブル名
     * @param meta
     * @return テーブルASエリアス、またはテンポラリーテーブル名
     */
    String getTableAsAliasOrTemporaryName(ITableDef.MetaData<?> meta) {
      if (isRemote(meta.$$) && !onEditingRemoteSelectSQL){
        return toTemporaryTableName(meta) ;
      } else {
        return meta.getTableAsAlias();
      }
    }
    
    /**
     * メタデータをもとにテンポラリーテーブル名を編集する.<br/>
     * @param meta
     * @return テンポラリーテーブル名
     */
    public String toTemporaryTableName(ITableDef.MetaData<?> meta)  {
      return "T" +  Integer.toHexString(this.hashCode()).toUpperCase() + "_" 
        +  (meta.hasAlias() ? meta.getAlias() : meta.getDBTableName(false));
    }
  }
  
  public enum UnionType {NONE, DEFAULT, ALL, MINUS};
  
  /**
   * SELECT文を構築するためクラス.<br/>
   * @author littlesoft
   *
   */
  static public class Select extends Query {
    
    protected Serializable[] values = new Serializable[0];
    protected Serializable[] tables = new Serializable[0];
    protected Serializable into = null;
    protected Serializable alias = null;
    ArrayList<Join> join = new ArrayList<Join>();
    protected Serializable[] orderBy = new Serializable[0];
    protected Serializable[] groupBy = new Serializable[0];
    protected Select unionSelect = null;
    protected UnionType unionType = UnionType.NONE;
    protected Condition where = null;
        
    /**
     * SELECTオブジェクトを作成します.<br/>
     */
    Select() {
      super();
    }
    
    /**
     * SELECTオブジェクトを作成します.<br/>
     * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
     * 省略すると[SELECT * FROM ...] と同じ意味になります。
     * @param columns ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
     */
    Select(Serializable...values) {
      super();
      values(values);
    }

    /**
     * 選択するデータ項目[SELECT ...]を指定します.<br/>
     * 省略すると[SELECT * FROM ...] と同じ意味になります。
     * @return Select
     */
    public Select values(Serializable...values) {
      setPrepared(false);
      this.values = values;
      int i = 0;
      for (Serializable  c : this.values) {
        if (c instanceof Select) {
          ((Select)c).setParent(this);
        } else if (c instanceof Token) {
          Token t = ((Token)c).getFirst();
          t.setQuery(this);
          this.values[i] = t;
        }
        i++;
      }
      return this;
    }

    
    /**
     * SELECT結果の転送先ダミーエンティティオブジェクトを指定します.<br/>
     * @param table
     * @return Select
     */
    public Select into(Serializable table) {
      setPrepared(false);
      Serializable entity = null;
      if (table instanceof EntityList) {
        entity = ((EntityList)table).getTable().$$;
      } else {
        entity = table;
      }
      if (isEntityObject(entity) && !isUsingEntity(entity)) {
        useDummy(entity);
      }
      into = table;
      return this;
    }

    /**
     * サブクエリーの別名[ (SELECT...) AS alias  ]を指定します.<br/>
     * @param alias ダミーエンティティオブジェクト
     * @return Select
     */
    public Select alias(Serializable alias) {
      setPrepared(false);
      if (isEntityObject(alias) && !isUsingEntity(alias)) {
        useDummy(alias);
      }
      this.alias = alias;
      return this;
    }
    
    String getAliasName() {
      Select sel = this;
      Serializable result = null;
      while (sel!=null) {
        if (result==null && sel.alias!=null) result = sel.alias;
        sel = sel.unionSelect;
      }
      if (result!=null && isUsingEntity(result)) {
        return getTableOrAliasName(result);
      } else {
        return null;
      }
    }

    /**
     * [ FROM ... ]を指定します.<br/>
     * @param tables ダミーエンティティオブジェクト あるいはサブクエリー(入れ子Select)
     * @return Select
     */
    public Select from(Serializable... tables) {
      setPrepared(false);
      this.tables = tables;
      int i = 0;
      for (Serializable t : tables) {
        if (t instanceof Select) {
          this.tables[i] = ((Select)t).getRoot();
          ((Select)this.tables[i]).setParent(this);
        } else if (t instanceof Token) {
          Token e = ((Token)t).getFirst();
          e.setQuery(this);
          this.tables[i] = e;
        } else if (isEntityObject(t) && !isUsingEntity(t)) {
          useDummy(t);
        }
        i++;
      }
      return this;
    }

    /**
     * SQL条件式[ WHERE ... ] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<SQL.Select> where(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition<SQL.Select>(this, null, nestedCondition);
      return new Condition<SQL.Select>(this, where);
    }

    /**
     * SQL条件式[ WHERE NOT(nestedCondition)] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<SQL.Select> whereNot(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition<SQL.Select>(this, null, nestedCondition);
      where.negative = true;
      return new Condition<SQL.Select>(this, where);
    }
    /**
     * SQL条件式[ WHERE ... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<SQL.Select,?> where(Serializable object) {
      setPrepared(false);
      where = new Condition<SQL.Select>(this, null);
      return where.$if(object);
    }
    
    /**
     * SQL条件式[ WHERE NOT(... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<SQL.Select,?> whereNot(Serializable object) {
      setPrepared(false);
      where = new Condition<SQL.Select>(this, null);
      return where.$not(object);
    }
    
    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであればSQL条件式[ WHERE ... ] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<SQL.Select> whereEvaluateNonNulls(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition<SQL.Select>(this, null, nestedCondition);
      where.setEvaluateNonNulls(true);
      return new Condition<SQL.Select>(this, where);
    }

    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであればSQL条件式[ WHERE ... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<SQL.Select,?> whereEvaluateNonNulls(Serializable object) {
      setPrepared(false);
      where = new Condition<SQL.Select>(this, null);
      where.setEvaluateNonNulls(true);
      return where.$if(object);
    }

    /**
     * 実行の際、動的に設定される条件式のキーを設定します.<br/>
     * @param String 条件式キー
     * @return Condition
     */
    public Condition<SQL.Select> whereDynamicCondition(String key) {
      setPrepared(false);
      where = new DynamicCondition<SQL.Select>(this, null, key);
      return new Condition<SQL.Select>(this, where);
    }
    /**
     * このSELECT中で主要と思われるエンティティを選択します.<br/>
     * @param includeIntoEntity
     * @return DBTableDef
     */
    public ITableDef.MetaData getPrimaryEntity(boolean includeIntoEntity) {
      //includeIntoEntityの場合 INTOで指定されたEntityはPrimaryです。
      if (includeIntoEntity && into!=null) {
        if (isEntityObject(into) && isUsingEntity(into)) {
          return getRoot().entityToMeta.get(into);
        }
      }
      //selectリストに出現するEntityがひとつであればPrimaryです。
      ArrayList<ITableDef.MetaData> list = new ArrayList<ITableDef.MetaData>();
      for (Serializable c : values) {
        if (getRoot().entityFieldToMeta.containsKey(c)) {
          ITableDef.MetaData e = getRoot().entityFieldToMeta.get(c);
          if (!list.contains(e)) {
            list.add(e);
          }
        }
      }
      if (list.size()==1) {
        return list.get(0);
      }
      //fromリストに出現するEntityがひとつであればPrimaryです。
      list.clear();
      for (Serializable t : tables) {
        if (isEntityObject(t) && isUsingEntity(t)) {
          ITableDef.MetaData e = getRoot().entityToMeta.get(t);
          if (!list.contains(e)) {
            list.add(e);
          }
        }
      }
      //最終的に候補がみつからない場合はINTO先をPrimaryにする
      if (list.size()==0 && into!=null) {
        if (isEntityObject(into) && isUsingEntity(into)) {
          return getRoot().entityToMeta.get(into);
        }
      }
      if (list.size()==1) {
        return list.get(0);
      }
      throw new RuntimeException("Please specify the entity by INTO.");
    }
    
    /**
     * [ INNER JOIN table ]を指定します.<br>
     * @param table
     * @return Join
     */
    public Join innerJoin(Serializable table) {
      return addJoin("INNER", table, null);
    }
    
    /**
     * [ LEFT OUTER JOIN table ]を指定します.<br>
     * @param table
     * @return Join
     */
    public Join leftOuterJoin(Serializable table) {
      return addJoin("LEFT OUTER", table, null);
    }

    /**
     * [ RIGHT OUTER JOIN table ]を指定します.<br>
     * @param table
     * @return Join
     */
    public Join rightOuterJoin(Serializable table) {
      return addJoin("RIGHT OUTER", table, null);
    }

    protected Join addJoin(String option, Serializable table, Condition conditon) {
      setPrepared(false);
      if (isEntityObject(table) && !isUsingEntity(table)) {
        useDummy(table);
      }
      Join j = new Join(this, option, table, conditon);
      join.add(j);
      return j;
    }

    /**
     * [ ORDER BY ... ]を指定します.
     * @param orderBy ダミーエンティティオブジェクトのフィールドあるいはExpressionオブジェクト
     * @return Select
     */
    public Select orderBy(Serializable...orderBy) {
      setPrepared(false);
      this.orderBy = new Serializable[orderBy.length];
      int i = 0;
      for (Serializable order : orderBy){
        if (order instanceof Token) {
          Token o = ((Token)order).getFirst();
          o.setQuery(this);
          this.orderBy[i] = o;
        } else {
          this.orderBy[i] = order;
        }
        i++;
      }
      return this;
    }
    
    /**
     * [ GROUP BY ... ]を指定します.
     * @param groupBy ダミーエンティティオブジェクトのフィールドあるいはExpressionオブジェクト
     * @return Select
     */
    public Select groupBy(Serializable...groupBy) {
      setPrepared(false);
      this.groupBy = new Serializable[groupBy.length];
      int i = 0;
      for (Serializable group : groupBy){
        if (group instanceof Token) {
          Token o = ((Token)group).getFirst();
          o.setQuery(this);
          this.groupBy[i] = o;
        } else {
          this.groupBy[i] = group;
        }
        i++;
      }
      return this;
    }
    
    /**
     * [ UNION SELECT ... ]で連結するSELECT文を作成します.
     * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
     * 省略すると[SELECT * FROM ...] と同じ意味になります。<br/>
     * ※UNIONで連結されるSELECT文は同じ列の構成である必要があります。
     * @return Select
     */
    public Select unionSelect(Serializable...columns) {
      setPrepared(false);
      unionSelect = new Select(columns);
      unionSelect.unionType = UnionType.DEFAULT;
      unionSelect.setParent(this);
      return unionSelect;
    }
    /**
     * [ UNION ALL SELECT ... ]で連結するSELECT文を作成します.
     * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
     * 省略すると[SELECT * FROM ...] と同じ意味になります。<br/>
     * ※UNION ALLで連結されるSELECT文は同じ列の構成である必要があります。
     * @return Select
     */
    public Select unionAllSelect(Serializable...columns) {
      setPrepared(false);
      unionSelect = new Select(columns);
      unionSelect.unionType = UnionType.ALL;
      unionSelect.setParent(this);
      return unionSelect;
    }
    /**
     * [ MINUS SELECT ... ]で連結するSELECT文を作成します.
     * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
     * 省略すると[SELECT * FROM ...] と同じ意味になります。<br/>
     * ※MINUSで連結されるSELECT文は同じ列の構成である必要があります。
     * @return Select
     */
    public Select minusSelect(Serializable...columns) {
      setPrepared(false);
      unionSelect = new Select(columns);
      unionSelect.unionType = UnionType.MINUS;
      unionSelect.setParent(this);
      return unionSelect;
    }
    
    public HashMap<ITableDef, String> getRemoteQueries() {
      HashMap<ITableDef, String> remoteQueries = new HashMap<ITableDef, String>();
      for (Serializable table : tables) {
        if (isRemote(table) && !remoteQueries.containsKey(table)) {
          remoteQueries.put((ITableDef)table, null);
        }
      }
      for (Join j : join) {
        if (isRemote(j.table) && !remoteQueries.containsKey(j.table)) {
          remoteQueries.put((ITableDef)j.table, null);
        }
      }
      for (ITableDef remoteTable : remoteQueries.keySet()) {
        StringBuilder buff = new StringBuilder();
        onEditingRemoteSelectSQL = true;
        try {
          buff.append("SELECT * FROM ");
          buff.append(entityToMeta.get(remoteTable).getDBTableName(true));
          getExportableConditions(this.where, remoteTable, buff, true); 
          remoteQueries.put(remoteTable, buff.toString());
        } finally {
          onEditingRemoteSelectSQL = false;
        }
      }
      return remoteQueries;
    }
    
    
    public void dropRemoteSnapshot(Connection conn) throws DataException {
      if (sqlContext.entityToRemote == null) return;
      for (Object object : sqlContext.entityToRemote.keySet()) {
        ITableDef remoteTable = (ITableDef)object;
        String dropSql = "DROP TABLE " + toTemporaryTableName(entityToMeta.get(remoteTable));
        UpdateSQL update = new UpdateSQL();
        update.setConnection(conn);
        update.setSQL(dropSql);
        try {
          update.executeUpdate(true, false);
          //System.out.println("");
        } catch (LSSQLException x) {}
      }
    }
    
    public void createRemoteSnapshot(Connection conn) throws DataException {
      if (sqlContext.idToRemote == null) return;
      if (sqlContext.idToRemote.size()>0) {
        
        HashMap<ITableDef, String> remoteQueries = getRemoteQueries();
        ArrayList<Table<?>> tempTables = new ArrayList<Table<?>>();
        
        for (ITableDef remoteEntity : remoteQueries.keySet()) {
          Remote remote = (Remote) sqlContext.entityToRemote.get(remoteEntity);
          if (remote == null) throw new DataException("Remote object does not exist.");
          Connection remoteConn = remote.getConnection();
          if (remoteConn == null) throw new DataException("Remote Database Connection does not exist.");

          SQLServerType type = null;
          if (remoteConn instanceof LSJsonConnection) {
            type = SQLServerType.LSJSON;
          } else {
            try {
              type = SQLServerType.checkSQLServerType(remoteConn.getMetaData().getDatabaseProductName());
            } catch (SQLException e) {
              throw new DataException(e);
            }
          }
          
          //テンポラリテーブル名のメタデータオブジェクトを作成する
          ITableDef.MetaData<?> meta = remoteEntity.createMetaData();
          meta.setDBSchema(null);
          meta.setDBTableName(toTemporaryTableName(entityToMeta.get(remoteEntity)));
          String option;
          switch(type) {
          //TODO 各種DBMS毎の対応
          case ORACLE:
            option = "TEMPORARY";
            break;
          default:
            option = "MEMORY";
          }
          //テンポラリテーブルをCREATE
          String createTempSql = meta.toCreateTableSQL(false, option);
          UpdateSQL update = new UpdateSQL();
          update.setConnection(conn);
          update.setSQL(createTempSql);
          try {
            update.executeUpdate(true, false);
          } catch (LSSQLException x) {}
          
          //リモートテーブルから読込、テンポラリテーブルへ書き出すテーブルオブジェクトを作成
          String selectRempteSql = remoteQueries.get(remoteEntity);
          Class entityType = remoteEntity.getClass();
          Table<?> temp = new Table();
          try {
            temp.setEntityType(entityType);
          } catch (Exception e) {
            throw new DataException(e);
          }
          //リモートへ接続するProvider
          ISQLDataProvider provider = getSQLContext().getProvider(remoteConn);
          temp.setDataProvider(provider);
          provider.setConnection(remoteConn);
          provider.setSQL(selectRempteSql);
          //テンポラリに書き出すResolver
          ISQLDataResolver resolver = getSQLContext().getResolver(conn);
          resolver.setTargetDBTable(meta.getDBTableName(false));
          resolver.setConnection(conn);
          temp.setDataResolver(resolver);
          tempTables.add(temp);
        }
        
        //リモートテーブルから読込＆テンポラリテーブルへ書き出しを実行する
        for (Table<?> temp : tempTables) {
          temp.getParams().copyValues(getParams());
          temp.activate();
          for (Row<?> r : temp.visibleRows()) {
            r.setStatus(Row.Status.CREATED);
          }
          temp.resolveChanges();
        }
      }
      
    }
    
  
    private void getExportableConditions(Condition<?> cond, Serializable remoteTable, StringBuilder buff, boolean init) {
      while (cond!=null) {
        if (cond.nestedCondition!=null) {
          getExportableConditions(cond.nestedCondition, remoteTable, buff, init); 
        } else if (cond.isExportable(remoteTable)) {
          boolean b = cond.withoutConjunction;
          cond.withoutConjunction = true;
          if (init) {
            buff.append(" WHERE "); 
            init = false;
          } else {
            buff.append(" AND "); 
          }
          buff.append(cond.toString());
          buff.append(cond.getNext().toString());
          cond.withoutConjunction = b;
        }
        Token t = cond.getNext();
        while (t != null && !(t instanceof Condition)) {
          t = t.getNext();;
        }
        cond = (Condition<?>)t;
      }
    }
    
    /**
     * SQL文を取得します.<br/>
     */
    @Override
    public String createSQL() {
      ArrayList<String> fromList = getFromList();
      ArrayList<String> joinList = getJoinList();
      ArrayList<String> colList  = getCoulmnList();
      ArrayList<String> orderList = getOrderByList();
      ArrayList<String> groupList = getGroupByList();
      
      StringBuilder s = new StringBuilder();
      s.append("SELECT");
      if (values.length == 0) {
        s.append(" * ");
      } else {
        s.append("\n");
      }
      writeList(colList, s);
      if (values.length >0)  s.append(indent());
      if (fromList.size()>0) {
        s.append("FROM\n");
        writeList(fromList, s);
      }
      if (joinList.size()>0) {
        for (String join : joinList) {
          s.append(" ");
          s.append(indent());
          s.append(join);
          s.append("\n");
        }
      }
      StringBuilder w = new StringBuilder();
      
      if (isUseAbsoluteFakeCondition()) {
        if (where==null) {
          s.append(indent() + "WHERE NULL IS NOT NULL\n");
        } else {
          s.append(indent() + "WHERE NULL IS NOT NULL AND (\n");
          where.writeSQL(w);
          if (!"".equals(w.toString().trim())) {
            s.append(indent() + " ");
            s.append(w.toString());
            s.append("\n");
          }
          s.append(")\n");
        }
      } else if (where!=null) {
        where.writeSQL(w);
        if (!"".equals(w.toString().trim())) {
          s.append(indent() + "WHERE\n");
          s.append(indent() + " ");
          s.append(w.toString());
          s.append("\n");
        }
      }

      if (groupList.size()>0) {
        s.append(indent() + "GROUP BY\n");
        writeList(groupList, s);
      }
      if (orderList.size()>0) {
        s.append(indent() + "ORDER BY\n");
        writeList(orderList, s);
      }
      
      if (unionSelect!=null) {
        switch(unionSelect.unionType) {
        case DEFAULT:
          s.append(indent() + "UNION\n" + indent());
          break;
        case ALL:
          s.append(indent() + "UNION ALL\n" + indent());
          break;
        case MINUS:
          s.append(indent() + "MINUS\n" + indent());
          break;
        }
        s.append(unionSelect.createSQL(level));
      }
      return s.toString();
    }
    
    /**
     * SELECTを実行し、結果リストを返します.<br/>
     * @param conn
     * @return List<T>
     * @throws DataException
     */
    public <T extends Serializable> List<T> execute(Connection conn)  throws DataException {
      return (List<T>) execute(conn, getSQLContext().typeParam, getSQLContext().getParams());
    }

     /**
     * SELECTを実行し、結果リストを返します.<br/>
     * @param conn
     * @return List<T>
     * @throws DataException
     */
    <T extends Serializable> List<T> execute(Connection conn, T[] typeParams, Params additionalParams)  throws DataException {
      return new EntityList(getTable(conn, typeParams, additionalParams));
    }
    
    <T extends Serializable> Table<T> getTable(Connection conn, T[] typeParams, Params additionalParams)  throws DataException {
      Table<T> tableResult = new Table<T>(typeParams);
      ISQLDataProvider provider = getSQLContext().getProvider(conn);
      tableResult.setDataProvider(provider);
      provider.setConnection(conn);
      provider.setSQL(getSQL(false));
      tableResult.getParams().copyValues(getParams());
      tableResult.activate();
      return tableResult;
    }
    
    @Override
    int execute(Connection conn, boolean pendingCommit, boolean reuseStatement) throws DataException {
      return 0;
    }
    
    /**
     * 現在のパラメータとその値をクリアした後、SELECT文を解析し、必要なパラメータを再設定します.<br/>
     * @param  true:evaluateNonNullsの条件を必ず含む
     * @return Select
     **/
    public Select prepare() {
      _prepare(true);
      return this;
    }

    /**
     * 現在のパラメータとその値をクリアした後、SELECT文を解析し、必要なパラメータを再設定します.<br/>
     * @param  true:evaluateNonNullsの条件を必ず含む
     * @return Select
     **/
    public Select prepare(boolean includeNullParamCondition) {
      _prepare(includeNullParamCondition);
      return this;
    }
    
    ArrayList<String> getCoulmnList() {
      ArrayList<String> colList = new ArrayList<String>();
      for (Serializable col : values) {
        if (getRoot().entityToMeta.containsKey(col)) {
          //SELECT TBL.* のケース
          colList.add(indent() + getRoot().entityToMeta.get(col).getTableOrAliasName() + ".*");
        } else if (col instanceof Select) {
          //入れ子SELECT
          Select sel = (Select)col;
          sel.setParent(this);
          colList.add(indent() + "(" + sel.createSQL(level+1) + ") " + getColumnName(sel.into));
        } else if (col instanceof Expression) {
          //式
          Expression exp = (Expression)col;
          if (isUsingEntity(exp.source)) {
            if (!isEntityObject(exp.dest) && !(exp.dest instanceof EntityList))
              throw new RuntimeException("Please specify the entity.");
            colList.add(indent() + getRoot().entityToMeta.get(exp.source).getTableOrAliasName() + ".*");
            into(exp.dest);
          } else {
            exp.setQuery(this);
            StringBuilder s = new StringBuilder();
            exp.writeSQL(s);
            colList.add(indent() + s.toString());
          }
        } else {
          try {
            colList.add(indent() + getColumnOrParameterName(this, col, false));
          } catch (NullParameterNotifycation e) {}
        }
      }
      return colList;
    }
    
//    public String getEditedTableAsAlias(Serializable table) {
//      ITableDef.MetaData<?> meta = getRoot().entityToMeta.get(table);
//      return getEditedTableAsAlias(meta);
//    }
//    
//    public String getEditedTableAsAlias(ITableDef.MetaData<?> meta) {
//      if (isRemote(meta.$$)){
//        /* REMOTE */
//        return toTemporaryTableName(meta) 
//                  + (meta.hasAlias() ? (" AS " + meta.getAlias()) : "");
//      } else {
//        return meta.getTableAsAlias();
//      }
//    }
//    public String toTemporaryTableName(ITableDef.MetaData<?> meta)  {
//      return "TEMP_" + meta.getDBTableName(false) + "" + this.hashCode();
//    }
//    
    ArrayList<String> getFromList() {
      ArrayList<String> fromList = new ArrayList<String>();
      for (Serializable table : tables) {
        if (getRoot().entityToMeta.containsKey(table)) {
          fromList.add(indent() + getTableAsAliasOrTemporaryName(table));
        } else if (table instanceof Select) {
          Select sel = (Select)table;
          sel.setParent(this);
          sel.level = level + 1;
          String alias = sel.getAliasName();
          fromList.add(indent() + "(" + ((Select)table).createSQL() + ") AS " + 
            ((alias!=null) ? alias : sel.getPrimaryEntity(true).getTableOrAliasName()) );
        } else if (table instanceof String){
          fromList.add(indent() + (String)table);
        } else {
          throw new RuntimeException("invalid description [from]:" + table.toString());
        }
      }
      if (fromList.size()==0) fromList.add(getTableAsAliasOrTemporaryName(this.getPrimaryEntity(true)));
      return fromList;
    }
    
    ArrayList<String> getJoinList() {
      ArrayList<String> joinList = new ArrayList<String>();
      for (Join j : join) {
        
        /** REMOTE **/
        joinList.add(j.toString());
        
      }
      return joinList;
    }

    ArrayList<String> getOrderByList() {
      ArrayList<String> orderByList = new ArrayList<String>();
      for (Serializable order : orderBy) {
        if (order instanceof Expression) {
          if (!((Expression)order).expressionType.equals(Expression.Type.ASCENDING) &&
              !((Expression)order).expressionType.equals(Expression.Type.DESCENDING)) {
            throw new RuntimeException("orderByが不正です。");
          }
          StringBuilder s = new StringBuilder();
          ((Expression)order).writeSQL(s);
          orderByList.add(indent() + s.toString());
        } else if (isUsingField(order)) {
          try {
            orderByList.add(indent() + getColumnOrParameterName(this, order, false));
          } catch (NullParameterNotifycation e) {}
        }
      }
      return orderByList;
    }

    ArrayList<String> getGroupByList() {
      ArrayList<String> groupByList = new ArrayList<String>();
      for (Serializable group : groupBy) {
        if (group instanceof Expression) {
          StringBuilder s = new StringBuilder();
          ((Expression)group).writeSQL(s);
          groupByList.add(indent() + s.toString());
        } else if (isUsingField(group)) {
          try {
            groupByList.add(indent() + getColumnOrParameterName(this, group, false));
          } catch (NullParameterNotifycation e) {}
        }
      }
      return groupByList;
    }

    @Override
    void setParent(Query parent) {
      super.setParent(parent);
      if (unionSelect!=null) {
        unionSelect.setParent(this);
      }
    }


    
  }
  

  /**
   * 
   * @author littlesoft
   *
   */
  public static class NestedJoin extends Select {
    
    
    public NestedJoin(Serializable table) {
      from(new Serializable[]{table});
    }
    
    public void writeSQL(StringBuilder builder, int level) {
      this.level = level;
      builder.append(getFromList().get(0)+ "\n");
      for (Join j : join) {
        builder.append(indent() + " " + j.toString() + "\n");
      }
    }
  }
  

  /**
   * SQLのJOINを表現するクラス
   * @author littlesoft
   */
  public static class Join extends Token {
    
    private String option;
    private Serializable table;
    private Condition condition = null;
    
    protected Join(Select select, String option, Serializable table, Condition condition) {
      super(select, null);
      this.option = option;
      this.table = table;
      if (condition!=null) on(condition);
    }
    @Override
    protected void setQuery(Query parent) {
      if (condition!=null) condition.setQuery(parent);
      if (table instanceof NestedJoin) {
        ((NestedJoin)table).setParent(parent);
      }
      super.setQuery(parent);
    }
    /**
     * JOINする条件式を指定します.<br/>
     * @param object ダミーエンティティオブジェクトのフィールド
     * @return Comparer
     */
    public Comparer<Select,?> on(Serializable object) {
      condition = new Condition(getQuery(), null);
      return condition.$if(object);
    }

    @Override
    public String toString() {
      StringBuilder s = new StringBuilder();
      s.append(option);
      s.append(" JOIN");
      if (table instanceof NestedJoin) {
        //ネストしたJOIN
        NestedJoin nestJoin = (NestedJoin)table;
        nestJoin.setParent(getQuery());
        s.append(" (\n");
        nestJoin.writeSQL(s, getQuery().level + 1);
        s.append(getQuery().indent() + " )");
      } else if (getQuery().isUsingEntity(table)) {
        s.append(" ");
        s.append(getQuery().getTableAsAliasOrTemporaryName(table));
      } else {
        s.append(" ");
        s.append(table.toString());
      }
      s.append(" ON (");
      condition.writeSQL(s);
      s.append(")");
      return s.toString();
    }
  }
  
  static abstract class UpdateQueryBase extends Query {
    
    Serializable entity = null;
    Expression[] expressions = new Expression[0];
    UpdateSQL updateSQL = null;
    
    UpdateQueryBase(Serializable entity) {
      useDummy(entity);
      this.entity = entity;   
    }
  
    /**
     * SQLを実行します。</br>
     * @param conn
     * @throws DataException
     */
    @Override
    public int execute(Connection conn, boolean pendingCommit, boolean reuseStatement) throws DataException {
      updateSQL = new UpdateSQL();
      updateSQL.setSQL(getSQL(false));
      updateSQL.copyValues(getParams());
      updateSQL.setConnection(conn);
      try {
        return updateSQL.executeUpdate(pendingCommit, reuseStatement);
      } finally {
        if (!reuseStatement) updateSQL = null;
      }
    }
    

  }
  
  /**
   * INSERT文を構築するオブジェクト
   * @author littlesoft
   *
   */
  static public class Insert extends UpdateQueryBase {
    
    Select select = null;
    
    Insert(Serializable entity) {
      super(entity);
    }
    
    /**
     * [ VALUES (... ]を指定します.<br/>
     * 引数には $(field).accept(":param")または$(field).accept( $(field).add(1) )")の様に
     * Expressionオブジェクトを指定します.<br/>
     * fieldには、更新対象エンティティダミーオブジェクトのフィールドを指定します.<br/>
     * @param expressions Expressionオブジェクトの配列
     * @return Update
     */
    public Insert values(Expression...expressions) {
      setPrepared(false);
      this.expressions = expressions;
      int i = 0;
      for (Expression  c : expressions) {
        Token t = c.getFirst();
        t.setQuery(this);
        this.expressions[i] = (Expression)t;
        i++;
      }
      return this;
    }
    
    /**
     * INSERTする内容を取得するSELECTオブジェクトを作成します。.<br/>
     * expressionsには選択するデータ項目を以下のように指定します.<br/>
     * insert().select(\n
     * &nbsp;&nbsp;$(employee.empId).into($$.id), ・・・ <br/>
     * @param expressions ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
     */
    public Select select(Expression...expressions) {
      setPrepared(false);
      Serializable[] exp = new Expression[expressions.length];
      int i = 0;
      for (Expression  c : expressions) {
        Token t = c.getFirst();
        t.setQuery(this);
        exp[i] = (Expression)t;
        i++;
      }
      select = new Select(exp);
      select.setParent(this);
      select.setSQLContext(getSQLContext());
      return select;
    }
    
    private void extractColumnAndValues(ArrayList<String> columnList, ArrayList<String> valueList, Expression exp, Expression.Type exType) {
      Serializable field = exp.source;
      switch(exType) {
      case INTO:
        Expression lastExp = exp;
        Serializable dest = null;
        while(lastExp!=null) {
          if (lastExp.expressionType.equals(Expression.Type.INTO)) {
            dest = lastExp.dest;
            break;
          }
          lastExp = ((Expression)exp.getNext());
        }
        if (dest!=null) {
          columnList.add(getColumnName(dest));
        } else {
          throw new RuntimeException("an invalid expression is specified [" + exp.toString() + "]");
        }
        break;
      case ACCEPT:
        
        if (isUsingField(field) && exp.expressionType.equals(exType) &&
            getRoot().entityFieldToMeta.get(field).$$==entity) {
          columnList.add(getColumnName(field));
          valueList.add(exp.toString(exp.dest));
        } else {
          throw new RuntimeException("an invalid expression is specified [" + exp.toString() + "]");
        }
        break;
      }
      
      
      
    }
    /**
     * SQL文を取得します.<br/>
     */
    @Override
    public String createSQL() {
      ArrayList<String> columnList = new ArrayList<String>();
      ArrayList<String> valueList = new ArrayList<String>();
      if (select != null) {
        for ( Serializable object : select.values) {
          if (!(object instanceof Expression)) continue;
          Expression exp = (Expression)object;
          extractColumnAndValues(columnList, valueList, exp, Expression.Type.INTO);
        }
      } else {
        if (expressions.length>0) {
          for ( Expression exp : expressions ) {
            exp.setQuery(this);
            extractColumnAndValues(columnList, valueList, exp, Expression.Type.ACCEPT);
          }
        } else {
          List<Column> cols = getRoot().entityToMeta.get(entity).getColumns();
          for (Column col : cols) {
            if (col.isReadOnly()) continue;
            columnList.add(col.getDBColumnName());
            valueList.add(":" + col.getName());
            getRoot().addParameter(col.getName(), col.getType());
          }
        }
      }
      StringBuilder builder = new StringBuilder();
      builder.append(indent() + "INSERT INTO ");
      builder.append(getDBTableName(entity));
      if (select!=null) {
        if (columnList.size()>0) {
          builder.append(" (\n");
          writeList(columnList, builder);
          builder.append(indent() + ")\n");
        }
        builder.append(indent() + " " + select.createSQL(this.level + 1));
      } else {
        if (columnList.size()>0) {
          builder.append("(\n");
          writeList(columnList, builder);
          builder.append(indent() + ")\n");
        }
        if (valueList.size()>0) {
          builder.append(indent() + "VALUES(\n");
          writeList(valueList, builder);
          builder.append(indent() + ")\n");
        }
      }
      return builder.toString();
    }

    
    /**
     * 現在のパラメータとその値をクリアした後、UPDATE文を解析し、必要なパラメータを再設定します.<br/>
     * @return Update
     */
    public Insert prepare() {
      _prepare(true);
      return this;
    }

    /**
     * 現在のパラメータとその値をクリアした後、UPDATE文を解析し、必要なパラメータを再設定します.<br/>
     * @param  true:evaluateNonNullsの条件を必ず含む
     * @return Update
     */
    public Insert prepare(boolean includeNullParamCondition) {
      _prepare(includeNullParamCondition);
      return this;
    }
  }
  
  /**
   * UPDATE文を構築するオブジェクト
   * @author littlesoft
   *
   */
  static public class Update extends UpdateQueryBase {
    
    private Condition where = null;
    
    Update(Serializable entity) {
      super(entity);
    }
    
    /**
     * [ SET ... ]を指定します.<br/>
     * 引数には $(field).accept(":param")または$(field).accept( $(field).add(1) )")の様に
     * Expressionオブジェクトを指定します.<br/>
     * fieldには、更新対象エンティティダミーオブジェクトのフィールドを指定します.
     * @param expressions Expressionオブジェクトの配列
     * @return Update
     */
    public Update set(Expression...expressions) {
      setPrepared(false);
      this.expressions = expressions;
      int i = 0;
      for (Expression  c : expressions) {
        Token t = c.getFirst();
        t.setQuery(this);
        this.expressions[i] = (Expression)t;
        i++;
      }
      return this;
    }
    
    
    /**
     * 実行の際、動的に設定される条件式のキーを設定します.<br/>
     * @param String 条件式キー
     * @return Condition
     */
    public Condition<Update> whereDynamicCondition(String key) {
      setPrepared(false);
      where = new DynamicCondition<Update>(this, null, key);
      return new Condition<Update>(this, where);
    }
    
    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであればSQL条件式[ WHERE ... ] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<SQL.Update> whereEvaluateNonNulls(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition<SQL.Update>(this, null, nestedCondition);
      where.setEvaluateNonNulls(true);
      return new Condition<SQL.Update>(this, where);
    }

    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであればSQL条件式[ WHERE ... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<SQL.Update,?> whereEvaluateNonNulls(Serializable object) {
      setPrepared(false);
      where = new Condition<SQL.Update>(this, null);
      where.setEvaluateNonNulls(true);
      return where.$if(object);
    }
    /**
     * SQL条件式[ WHERE ... ] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<Update> where(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition(this, null, nestedCondition);
      return new Condition(this, where);
    }

    /**
     * SQL条件式[ WHERE ... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<Update,?> where(Serializable object) {
      setPrepared(false);
      where = new Condition(this, null);
      return where.$if(object);
    }
    /**
     * SQL文を取得します.<br/>
     */
    @Override
    public String createSQL() {
      StringBuilder builder = new StringBuilder();
      builder.append(indent() + "UPDATE ");
      builder.append(getDBTableName(entity));
      builder.append(" SET\n");
      ArrayList<String> colList = new ArrayList<String>();
      for ( Expression exp : expressions ) {
        exp.setQuery(this);
        StringBuilder s = new StringBuilder();
        exp.writeSQL(s);
        colList.add(indent() + s.toString());
      }
      writeList(colList, builder);
      if (where!=null) {
        builder.append(indent() + "WHERE\n");
        builder.append(indent() + " ");
        where.writeSQL(builder);
        builder.append("\n");
      }
      return builder.toString();
    }

    /**
     * 現在のパラメータとその値をクリアした後、UPDATE文を解析し、必要なパラメータを再設定します.<br/>
     * @return Update
     */
    public Update prepare() {
      _prepare(true);
      return this;
    }
    
    /**
     * 現在のパラメータとその値をクリアした後、UPDATE文を解析し、必要なパラメータを再設定します.<br/>
     * @param  true:evaluateNonNullsの条件を必ず含む
     * @return Update
     */
    public Update prepare(boolean includeNullParamCondition) {
      _prepare(includeNullParamCondition);
      return this;
    }
  }
  
  /**
   * UPDATE文を構築するオブジェクト
   * @author littlesoft
   *
   */
  static public class Delete extends UpdateQueryBase {
    
    private Condition where;
    
    Delete(Serializable entity) {
      super(entity);
    }
    
    /**
     * 実行の際、動的に設定される条件式のキーを設定します.<br/>
     * @param String 条件式キー
     * @return Condition
     */
    public Condition<Delete> whereDynamicCondition(String key) {
      setPrepared(false);
      where = new DynamicCondition<Delete>(this, null, key);
      return new Condition<Delete>(this, where);
    }
    
    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであればSQL条件式[ WHERE ... ] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<SQL.Delete> whereEvaluateNonNulls(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition<SQL.Delete>(this, null, nestedCondition);
      where.setEvaluateNonNulls(true);
      return new Condition<SQL.Delete>(this, where);
    }

    /**
     * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
     * 実行時パラメータがNOT NULLであればSQL条件式[ WHERE ... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<SQL.Delete,?> whereEvaluateNonNulls(Serializable object) {
      setPrepared(false);
      where = new Condition<SQL.Delete>(this, null);
      where.setEvaluateNonNulls(true);
      return where.$if(object);
    }
    /**
     * SQL条件式[ WHERE ... ] を指定します。
     * @param nestedCondition 入れ子条件式
     * @return Condition
     */
    public Condition<Delete> where(Condition nestedCondition) {
      setPrepared(false);
      where = new Condition(this, null, nestedCondition);
      return new Condition(this, where);
    }

    /**
     * SQL条件式[ WHERE ... ] を指定します。
     * @param object ダミーエンティティオブジェクトのフィールド、固定値、パラメータ
     * @return Comparer
     */
    public Comparer<Delete,?> where(Serializable object) {
      setPrepared(false);
      where = new Condition(this, null);
      return where.$if(object);
    }
    /**
     * SQL文を取得します.<br/>
     */
    @Override
    public String createSQL() {
      StringBuilder builder = new StringBuilder();
      builder.append(indent() + "DELETE FROM ");
      builder.append(getDBTableName(entity));
      if (where!=null) {
        builder.append("\n");
        builder.append(indent() + "WHERE\n");
        builder.append(indent() + " ");
        where.writeSQL(builder);
        builder.append("\n");
      }
      return builder.toString();
    }

    /**
     * 現在のパラメータとその値をクリアした後、UPDATE文を解析し、必要なパラメータを再設定します.<br/>
     * @return Update
     */
    public Delete prepare() {
      _prepare(true);
      return this;
    }
    
    /**
     * 現在のパラメータとその値をクリアした後、UPDATE文を解析し、必要なパラメータを再設定します.<br/>
     * @param  true:evaluateNonNullsの条件を必ず含む
     * @return Update
     */
    public Delete prepare(boolean includeNullParamCondition) {
      _prepare(includeNullParamCondition);
      return this;
    }
  }
  

  /**
   * コネクションをもとに適切なデータプロバイダーのインスタンスを生成する.<br/>
   * @param conn
   * @return SQLDataProviderオブジェクト
   * @throws DataException
   */
  public ISQLDataProvider getProvider(Connection conn) throws DataException {
    //コネクションをもとにデータプロバイダー実装クラスを決定する.
    if (conn instanceof ISQLTableStorage) {
      return ((ISQLTableStorage)conn).getProvider();
    } else {
      //ディフォルトのデータプロバイダー
      return new SQLDataProviderImpl();
    }
  }
  
  /**
   * コネクションをもとに適切なデータリゾルバーのインスタンスを生成する.<br/>
   * @param conn
   * @return SQLDataResolverオブジェクト
   * @throws DataException
   */
  public ISQLDataResolver getResolver(Connection conn) throws DataException {
    //コネクションをもとにデータリゾルバー実装クラスを決定する.
    if (conn instanceof ISQLTableStorage) {
      return ((ISQLTableStorage)conn).getResolver();
    } else {
      //ディフォルトのデータリゾルバー
      return new SQLDataResolverImpl(); 
    }
  }
  
  /**
   * SELECTを実行し、結果リストを指定したTableへ格納します。<br/>
   * 
   * @param selectConn SELECT実行コネクション
   * @param targetTable 格納先Table
   * @param insertConn INSERT実行コネクション
   * @throws DataException 例外発生時
   */
  public void selectCopyToTable(Connection selectConn, WebSQLTable<?> targetTable, Connection insertConn) throws DataException {
    // 元のProvider
    ISQLDataProvider orgProvider = (ISQLDataProvider)targetTable.getDataProvider();
    // 元のResolver
    ISQLDataResolver orgResolver = (ISQLDataResolver)targetTable.getDataResolver();
    // SELECT文発行元へ接続するProvider
    ISQLDataProvider provider = getProvider(selectConn);
    targetTable.setDataProvider(provider);
    provider.setConnection(selectConn);
    provider.setSQL(getSQL());
    // コピー対象に書き出すResolver
    ISQLDataResolver resolver = getResolver(insertConn);
    resolver.setTargetDBSchema(targetTable.getSQLSelect().getPrimaryEntity(false).getDBSchema());
    resolver.setTargetDBTable(targetTable.getSQLSelect().getPrimaryEntity(false).getDBTableName(false));
    resolver.setConnection(insertConn);
    targetTable.setDataResolver(resolver);
    
    targetTable.getParams().copyValues(getParams());
    targetTable.activate();
    for (Row<?> r : targetTable.visibleRows()) {
      r.setStatus(Row.Status.CREATED);
    }

    targetTable.setDataProvider(orgProvider);
    targetTable.setDataResolver(orgResolver);
  }

}
