/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.sql;

import java.math.BigDecimal;
import java.sql.DriverManager;

import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Column.Nullable;
import jp.littlesoft.data.ITableDef.TableDef;

@SuppressWarnings("serial")
public class SchemaUpdatorTest {
  
  @TableDef(owner="SA", schema="TESTNEW")
  static public class UserGroup implements ITableDef {
    @ColumnDef(caption="グループID", nullable=Nullable.FALSE )
    public Integer id;
    @ColumnDef(caption="グループ名", defaultValue="[設定してください]", length=120, nullable=Nullable.FALSE)
    public String name;
    
    @ColumnDef(precision=15, scale=2 , defaultValue="0.12")
    public BigDecimal number;

    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<UserGroup>() {{
        addPrimaryKey($$.id);
      }};
    }
  }
  
  @TableDef(owner="SA", schema="TESTNEW")
  static public class User implements ITableDef {
    @ColumnDef(caption="ユーザーID", nullable=Nullable.FALSE )
    public Long id;
    @ColumnDef(caption="ユーザー名", length=80, nullable=Nullable.FALSE)
    public String name;
    @ColumnDef(oldName="GROUP_ID", caption="グループID", nullable=Nullable.TRUE )
    public Integer gid;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<User>() {{
        addPrimaryKey($$.id);
        addForeginKey($$.gid)
          .references(UserGroup.class)
          .onDelete(FKOption.RESTRICT)
          .onUpdate(FKOption.SET_NULL)
          .usingIndex(true);
        addIndex($$.name);
      }};
    }
  }
  
  @TableDef(owner="SA", schema="TESTNEW")
  static public class Base implements ITableDef {
    @ColumnDef(caption="登録日時" , formatPattern="GGyy/MM/dd hh:mm:ss.SSS")
    public java.sql.Timestamp createDatetime;
    @ColumnDef(caption="更新日時" , formatPattern="GGyy/MM/dd hh:mm:ss.SSS")
    public java.sql.Timestamp updateDatetime;
    @SuppressWarnings("unchecked")
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData(this.getClass());
    }
  }
  
  @TableDef(owner="SA", schema="TESTNEW")
  static public class Pref extends Base {

    @ColumnDef(nullable=Nullable.FALSE)
    public Integer code;
    @ColumnDef(length=120, nullable=Nullable.FALSE)
    public String name;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<Pref>() {{
        addPrimaryKey($$.code);
      }};
    }
  }

  @TableDef(owner="SA", schema="TESTNEW")
  static public class City extends Base {

    @ColumnDef(nullable=Nullable.FALSE)
    public Integer prefCode;
    @ColumnDef(nullable=Nullable.FALSE)
    public Short cityCode;
    @ColumnDef(length=100, nullable=Nullable.FALSE)
    public String name;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<City>() {{
        addPrimaryKey($$.prefCode, $$.cityCode);
        addForeginKey($$.prefCode).references(Pref.class)
          .onUpdate(FKOption.SET_NULL)
          .onDelete(FKOption.CASCADE)
          .usingIndex(false);
      }};
    }
  }

  @TableDef(owner="SA", schema="TESTNEW")
  static public class Address extends Base {

    @ColumnDef(nullable=Nullable.FALSE)
    public Short prefCode;
    @ColumnDef(nullable=Nullable.FALSE)
    public Short cityCode;
    @ColumnDef(nullable=Nullable.FALSE)
    public Short addressCode;
    @ColumnDef(length=120, nullable=Nullable.FALSE)
    public String newStreetName;
    @ColumnDef(length=80, nullable=Nullable.TRUE)
    public String buildingName;
    @ColumnDef(length=20)
    public String tel;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<Address>() {{
        addPrimaryKey($$.prefCode, $$.cityCode, $$.addressCode);
        addForeginKey($$.prefCode, $$.cityCode).references(City.class)
          .onUpdate(FKOption.RESTRICT)
          .onDelete(FKOption.CASCADE)
          .usingIndex(false);
        addIndex($$.newStreetName).unique(false);
        addIndex($$.buildingName).unique(false);
      }};
    }
  }
  /**
   * @param args
   */
  public static void main(String[] args) {
    try {
      System.out.println("@@@ strat @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      Class.forName("org.h2.Driver");
      java.sql.Connection conn = DriverManager.getConnection("jdbc:h2:file:C:/gomi/mydb2", "TNAGAO", "123");
      
//      new SchemaUpdator(conn).dropTables(
//        UserGroup.class,
//        User.class
//      );

      new SchemaUpdator(conn).createOrAlterTable(
        UserGroup.class,
        User.class
      );
      
//      updator.createOrReplaceTable(
//        Pref.class,
//        City.class,
//        Address.class
//      );
      
//      updator.createOrAlterTable(
//        Pref.class,
//        City.class,
//        Address.class
//      );

    } catch (Exception e) {
      e.printStackTrace();
    }
    System.out.println("@@@ exit @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
    System.exit(0);
  }

}
