/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.util.res;

import java.text.MessageFormat;
import java.util.*;

import jp.littlesoft.util.Const;
import jp.littlesoft.util.StringUtil;

/**
 * リソースから文字列の取得を行うためのクラス。<br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class MessageResource extends ResourceString {
  private Object[] messageArguments = null;

  /**
   * コンストラクタ。<br/>
   *
   * @param resourceClassName リソースのクラス名(nullの場合は"jp.littlesoft.utl.Msg"となります。)
   * @param resourceFieldName リソースのフィールド名
   * @param msgArgs 置換文字列の配列
   */
  public MessageResource(String resourceClassName, String resourceFieldName, Object[] msgArgs) {
    super();
    if (StringUtil.isNullString(resourceClassName)) {
      resourceClassName = "jp.littlesoft.util.Msg";
    }
    this.setResourceClassName(resourceClassName);
    this.setResourceFieldName(resourceFieldName);
    this.setMessageArguments(msgArgs);
  }
  
  /**
   * メッセージ中の{0},{1}などに対する置換文字列の配列を設定します。<br/>
   * 
   * @param msgArgs {0},{1}に対する置換文字列の配列
   */
  public void setMessageArguments(Object[] msgArgs) {
    this.messageArguments = msgArgs;
  }
  
  /**
   * メッセージ中の{0},{1}に対する置換文字列の配列を取得します。<br/>
   * 
   * @return {0},{1}に対する置換文字列の配列
   */
  public Object[] getMessageArguments() {
    return this.messageArguments;
  }
  
  /**
   * ロケールに応じたメッセージを取得します。<br/>
   * 
   * @param locale ロケール(nullの場合は、JVMのロケールが適用されます)
   * @return メッセージ
   */
  public String getMessage(Locale locale) {
    String msgFmt;
    try {
      msgFmt = this.getValue(locale);
    } catch (Exception ex) {
      ex.printStackTrace();
      msgFmt = null;
    }
    if (StringUtil.isNullString(msgFmt)) {
      return Const.NULLSTRING;
    } else {
      return MessageFormat.format(msgFmt, this.messageArguments);
    }
  }
  
  /**
   * リソースクラス名とフィールド名、ロケール、置換文字列を指定して、メッセージを取得します。<br/>
   * 
   * @param resourceClassName メッセージのリソースクラス名
   * @param resourceFieldName メッセージのフィールド名
   * @param msgArgs 置換文字列の配列
   * @param locale ロケール
   * @return メッセージ
   */
  public static String getMessage(String resourceClassName, String resourceFieldName, Object[] msgArgs, Locale locale) {
    MessageResource res = new MessageResource(resourceClassName, resourceFieldName, msgArgs);
    return res.getMessage(locale);
  }
  
}

