/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.littlesoft.data.Column;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Table;
import jp.littlesoft.data.exception.DataException;
import org.apache.velocity.VelocityContext;


@SuppressWarnings("serial")
public class DefaultDialog<E extends Serializable> extends SQLActionPage {

  protected HashMap<String, Serializable> args;
  private IDialogClient dialogClient;
  protected int buttonWidth = 70;
  protected String templateResource;
  protected String formStyle = "";
  protected String title = "****";
  private boolean autoClose = true;
  /**
   * ダイアログで使用するボタンの定義
   * @author littlesoft
   *
   */
  public enum Button{OK,CANCEL,YES,NO,OPEN,CLOSE,LOAD,SAVE,EXEC,CONFIRMED};
  static HashMap<Button,String[]> buttonValues = new HashMap<Button,String[]>() {{
    put(Button.OK,      new String[]{"OK",""});
    put(Button.CANCEL,  new String[]{"キャンセル",""});
    put(Button.YES,     new String[]{"はい",""});
    put(Button.NO,      new String[]{"いいえ",""});
    put(Button.OPEN,    new String[]{"開く",""});
    put(Button.CLOSE,   new String[]{"閉じる",""});
    put(Button.LOAD,    new String[]{"読込み",""});
    put(Button.SAVE,    new String[]{"保存",""});
    put(Button.EXEC,    new String[]{"実行",""});
  }};
  
  protected Button[] buttons;
  private WebDataTable<E> dataTable;
  private Serializable clientData = null; 
  private String onCloseScript = "";
  
  public DefaultDialog(E...e){
    dataTable = new WebDataTable<E>(e);
  }
  
  @SuppressWarnings("unchecked")
  public DefaultDialog(HttpServletRequest request, HttpServletResponse response, E...e) {
    super(request, response);
    setTemplateName("/common/" + DefaultDialog.class.getSimpleName() + ".html");
    dataTable = new WebDataTable<E>(e);
    setUseAjax(true);
    args = (HashMap<String, Serializable>)request.getAttribute(ActionPage.RA_PARENT_ARGS);
    if (args.containsKey("title")) {
      this.title = (String)args.get("title");
    }
    ActionPage parent = (ActionPage)request.getAttribute(ActionPage.RA_PARENT_PAGE);
    setParentPage(parent);
    if (getParentPage() == null) {
      throw new RuntimeException("Parent ActionPage is null.");
    } else if (getParentPage() instanceof IDialogClient) {
      dialogClient = (IDialogClient)getParentPage();
    } else {
      throw new RuntimeException("Parent ActionPage must implement IDialogClient.");
    }
    try {
      dataTable.activate();
      dataTable.appendRow();
//      acceptCleintData();
    } catch (DataException x) {
      throw new RuntimeException(x);
    }
  }
  
 static protected String getHTMLButton(Button button, int buttonWidth){
    return "<input type=\"button\" class=\"w_" + buttonWidth 
         + "\" value=\"" + buttonValues.get(button)[0] 
         + "\" onclick=\"onButtonClick({button:'" + button.toString() + "'});\" />" ;
  }
  
  /**
   * 2010.1.30 tnagao : このメソッドからテンプレートへ埋め込みたいが、うまくいかないので、現状べた書き  
   * @return
   */
  protected String getHeaderScript() {
    StringBuffer b = new StringBuffer();
    b.append("var hide,onButtonClick,onConfirmed,afterButtonClick;\n");
    b.append("function windowLoaded() {\n");
    b.append(  "parent.defaultDialog.setTitle('" + this.title + "');\n");
    b.append(  "hide = function(){parent.defaultDialog.hide();}\n");
    b.append(  "onButtonClick = function(b){ls.ajax('onButtonClick',afterButtonClick,b);}\n");
    b.append(  "onConfirmed = function(){ls.ajax('onButtonClick',afterButtonClick,{button:'" + Button.CONFIRMED.toString() + "'});}\n");
    b.append(  "afterButtonClick = {\n");
    b.append(    "asynchronous: 'false',\n");
    b.append(    "onComplete: function(response) {\n");
    b.append(      "var result = eval('(' + response.content + ')');\n");
    b.append(      "result(); \n");
    b.append(    "}\n");
    b.append(  "}\n");
    b.append("}\n");
    return b.toString();
  }

  static protected String jsShowInfo(String message) {
    return "function(){parent.msg.info('" + message + "',{confirmTrue:hide});}";
  }
 
  static protected String jsShowError(String message) {
    return "function(){parent.msg.error('" + message + "');}";
  }

  static protected String jsShowConfirm(String message) {
    return "function(){parent.msg.confrim('"  + message + "',{confirmTrue:onConfirmed});}";
  }
 
  /**
   * ダイアログボタンがクリックされた際のアクション
   * @param args
   * @throws Exception
   */
  public void onButtonClick(HashMap<String, Object> args) throws Exception {
    String stringResult = "";
    try {
      Button clicked = Button.valueOf((String)args.get("button"));
      switch (clicked) {
      case CANCEL:
      case CLOSE:
        //stringResult = (isAutoClose()) ? "function(){hide();}" : "";
        stringResult = "function(){" + onCloseScript + "hide();}";
        closeSubPage();
        break;
      case NO:
        break;
      default:
        if (!dataTable.checkRequest(request)) {
          throw new IDialogClient.Error(HTML.escape(dataTable.getErrorMessages(), false, true));
        }
        onValidate(dataTable.entity); 
        updateCleintData();
        String resultScript = dialogClient.evaluateDialogResult(this, clicked, clientData);
        stringResult = "function(){" 
          + ((resultScript==null) ? "" :  resultScript) 
          +  ((isAutoClose()) ? "hide();}" : "}");
        closeSubPage();
        break;
      }
    } catch (IDialogClient.Error clientError) {
      stringResult = jsShowError(clientError.getLocalizedMessage());
    } catch (IDialogClient.Confirm clientConfirm) {
      stringResult = jsShowConfirm(clientConfirm.getLocalizedMessage());
    }
    writeAjaxResponse(stringResult);
  }
  
  /**
   * ダイアログボタンがクリックされた際,データ内容の検証を実装する為のプロテクトメソッド.<br/>
   * 当クラスを継承したアプリケーションクラスでデータチェック等の処理を実装してください.<br/>
   * エラーが検出した際は、適切なメッセージとともにIDialogClient.Errorをスローしてください.
   * これによりディフォルトのエラーメッセージボックスがブラウザに表示されます.<br/>
   * また、ユーザーの確認（YES/NO）を必要とすろ場合はIDialogClient.Confirmをスローしてください.
   * これによりYES/NOの確認メッセージボックスがブラウザに表示されます.<br/>
   * @param data
   * @throws IDialogClient.Error
   * @throws IDialogClient.Confirm
   */
  protected void onValidate(E data) throws IDialogClient.Error, IDialogClient.Confirm {
    
  }
  
  protected void setButtons(Button...button) {
    buttons = button;
  }

  /**
   * ダイアログの呼び出しもとからデータを取得します.<br>
   * 継承クラスのコンストラクタで当メソッドを呼び出してください.<br/>
   * @throws DataException
   */
  @SuppressWarnings("unchecked")
  protected void acceptCleintData() throws DataException {
    clientData = dialogClient.getDialogClientData(this);
    if (clientData instanceof Table) {
      Table<?> t = (Table<?>)clientData;
      if (!t.isActive()) return;
      for (Column c : t.getColumnArray()) {
        String name = c.getName();
        if (t.getRow().isNull(name)) continue;
        setValue(dataTable, name, t.getRow().getValue(name));
      }
    } else if (clientData instanceof Map){
      Map<String, Serializable> map = (Map<String, Serializable>)clientData;
      for (String name : map.keySet()) {
        if (map.get(name)==null) continue;
        setValue(dataTable, name, map.get(name));
      }
    } else {
      ITableDef.Columns cols = new ITableDef.Columns(clientData.getClass());
      for (Column c : cols) {
        Serializable v = c.getEntityValue(clientData);
        if (v==null) continue;
        setValue(dataTable, c.getName(), v);
      }
    }
  }

  @SuppressWarnings("unchecked")
  protected void updateCleintData() throws DataException {
    if (clientData instanceof Table) {
      Table<?> t = (Table<?>)clientData;
      for (Column c : dataTable.getColumnArray()) {
        String name = c.getName();
        setValue(t, name, dataTable.getRow().getValue(name));
      }
    } else if (clientData instanceof Map){
      Map<String, Serializable> map = (Map<String, Serializable>)clientData;
      for (Column c : dataTable.getColumnArray()) {
        String name = c.getName();
        map.put(name, dataTable.getRow().getValue(name));
      }
    } else {
      ITableDef.Columns cols = new ITableDef.Columns(clientData.getClass());
      for (Column c : cols) {
        String name = c.getName();
        c.setEntityValue(clientData, dataTable.getRow().getValue(name));
      }
    }    
  }
  
  private void setValue(Table dest, String name, Serializable value) throws DataException {
    if (dest.getColumns().containsKey(name) && !dest.getColumn(name).isReadOnly()) {
      dest.getRow().setValue(name, value);
    }
  }
  
  @Override
  protected void onStartShow(VelocityContext velocityContext) throws Exception {
    velocityContext.put("title", title);
    velocityContext.put("headerScript", getHeaderScript());
    velocityContext.put("buttons", getButtons());
    velocityContext.put("formElements", getFormElements());
    super.onStartShow(velocityContext);
  }

  protected String getButtons() {
    StringBuffer buff = new StringBuffer();
    for (Button b : buttons) {
      buff.append(getHTMLButton(b, buttonWidth));
    }
    return buff.toString();
  }
  
  protected String getFormElements() throws Exception{
    return new HTML.FormTemplate(dataTable, formStyle).createSimple(0);
  }

  protected WebDataTable<E> getDataTable() {
    return dataTable;
  }

  protected void setDataTable(WebDataTable<E> dataTable) throws DataException {
    this.dataTable = dataTable;
    try {
      if (!this.getDataTable().isActive())
        this.getDataTable().activate();
      if (this.getDataTable().getRowCount()==0)
        this.getDataTable().appendRow();
    } catch (DataException e) {
      throw new RuntimeException(e);
    }
    clientData = dialogClient.getDialogClientData(this);
    acceptCleintData();
  }

  public String getTitle() {
    return title;
  }

  public void setTitle(String title) {
    this.title = title;
  }

  public IDialogClient getDialogClient() {
    return dialogClient;
  }

  protected HashMap<String, Serializable> getArgs() {
    return args;
  }

  protected void setArgs(HashMap<String, Serializable> args) {
    this.args = args;
  }

  protected int getButtonWidth() {
    return buttonWidth;
  }

  protected void setButtonWidth(int buttonWidth) {
    this.buttonWidth = buttonWidth;
  }

  protected String getTemplateResource() {
    return templateResource;
  }

  protected void setTemplateResource(String templateResource) {
    this.templateResource = templateResource;
  }

  protected String getFormStyle() {
    return formStyle;
  }

  protected void setFormStyle(String formStyle) {
    this.formStyle = formStyle;
  }

  public boolean isAutoClose() {
    return autoClose;
  }

  public void setAutoClose(boolean autoClose) {
    this.autoClose = autoClose;
  }

  public String getOnCloseScript() {
    return onCloseScript;
  }

  public void setOnCloseScript(String onCloseScript) {
    this.onCloseScript = onCloseScript;
  }
}
