/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import javax.servlet.http.HttpServletRequest;
import jp.littlesoft.sql.SQLTreeModel;

public class WebTreeModel extends SQLTreeModel implements ICheckableRequest {

  private static final long serialVersionUID = -586624234019609058L;
  private String cssClassNameTreeView = "lsTreeView";
  private String cssClassNameNodeExpanded = "lsTreeViewToggleOpen";
  private String cssClassNameNodeCollapsed = "lsTreeViewToggleClose";
  private String cssClassNameLabel = "lsTreeViewToggleLabel";
  private String cssClassNameLeaf  = "lsTreeViewItem";
  private String name;
  private String onSelectionMethodName = "doDefault";
  private String onSelectionCompleteAjaxHandlerName = null;
  private String selectedItemColor = "#6495ED";
  private boolean useExpandAction = false;
  private boolean useCollapseAction = false;
  /**
   * @return useExpandAction
   */
  public boolean isUseExpandAction() {
    return useExpandAction;
  }

  /**
   * @param useExpandAction セットする useExpandAction
   */
  public void setUseExpandAction(boolean useExpandAction) {
    this.useExpandAction = useExpandAction;
  }

  /**
   * @return useCollapseAction
   */
  public boolean isUseCollapseAction() {
    return useCollapseAction;
  }

  /**
   * @param useCollapseAction セットする useCollapseAction
   */
  public void setUseCollapseAction(boolean useCollapseAction) {
    this.useCollapseAction = useCollapseAction;
  }

  //protected String selectionNodeId = "";
  private String id = HTML.getRandomValue(this) + "_";
  public WebTreeModel() {
    super();
  }

  private String getToggle(boolean expanded) {
    return (expanded) ? cssClassNameNodeExpanded  : cssClassNameNodeCollapsed;
  }

  @Override
  public String toString() {
    return getHTML(name);
  }

  private String onSelectionName() {
    return name + "_onSelection";
  }

  public String getSelectionScript(Node node) {
    return  onSelectionName() + "('" + node.getId() + "')";
  }

  private String onExpandName() {
    return name + "_onExpand";
  }

  private String onCollapseName() {
    return name + "_onCollapse";
  }

  private String selected() {
    return name + "_selected";
  }

  private String defaultInformationName() {
    return name + "_defaultInformation";
  }

  private String onClickScript(Node node) {
    return "onclick=\"" + onSelectionName() + "('" + node.getId() + "')\"";
  }

//
//  public String getSelectionRequest() {
//    StringBuffer buff = new StringBuffer();
//    buff.append("<script type=\"text/javascript\"><!--\n");
//    buff.append("function " + onSelectionName() + "(node) {\n");
//    buff.append("  var labelId = 'label' + node;\n");
//    buff.append("  try {\n");
//    buff.append("    var oldId = 'label' + $('" + id + "').value;\n");
//    buff.append("    $(oldId).style.color = 'none';\n");
//    buff.append("  } catch (e) {};\n");
//    buff.append("  $('" + id + "').value = node;\n");
//    buff.append("  $(labelId).style.color = 'red';\n");
//    buff.append("  ls.ajax('" + onSelectionMethodName + "',");
//    if (onSelectionCompleteAjaxHandlerName != null ) {
//      buff.append(" " + onSelectionCompleteAjaxHandlerName + ");");
//      buff.append("}\n");
//    } else {
//      buff.append(" on_" + onSelectionName() + ");");
//      buff.append("}\n");
//      buff.append("var on_" + onSelectionName() + " = {");
//      buff.append(" asynchronous: 'false',");
//      buff.append(" onComplete: function(response){");
//      buff.append("   Element.Methods.update($('" + defaultInformationName() + "'), response." + name
//          + "_defaultInformation);");
//      buff.append(" }");
//      buff.append("}\n");
//    }
//
//
//    buff.append("//-->");
//    buff.append("</script>");
//    return buff.toString();
//  }



  /*
      var ajaxUpdateUITable = {
      asynchronous: 'false',
      onComplete: function(response){
        Element.Methods.update('message_table_body','');
        Element.Methods.update('dataCount',response.tableEmp.rowCount);
        Element.Methods.update('employee_table_body',response.content);
        if (response.tableEmp.hasError == true) {
          alert(response.tableEmp.errors);
        }
        $('btnFirst').disabled = !response.tableEmp.hasPreviousPage;
        $('btnPrev').disabled  = !response.tableEmp.hasPreviousPage;
        $('btnNext').disabled  = !response.tableEmp.hasNextPage;
        $('btnLast').disabled  = !response.tableEmp.hasNextPage;
        $('btnCheck').disabled = (response.tableEmp.rowCount==0);
        $('btnSave').disabled  = (response.tableEmp.rowCount==0);
      }
    }
   */

  public String getNodeId(Node node) {
    if (node==this.getRoot()) {
      return name;
    } else {
      return Long.toString(node.getId());
    }
  }

  private String toId(Node node, String prefix) {
    return " id=\"" + prefix + node.getId() + "\" ";
  }

  private String toLabelId(Node node) {
    return toId(node, "label");
  }

  private String toNodeId(Node node) {
    return  toId(node, "");
  }

  /**
   * TreeNode全体のHTMLを取得します.<br/>
   * @param domElementId
   * @return
   */
  public String getHTML(String domElementId) {
    StringBuffer buff = new StringBuffer();
    buff.append("<div id=\"");
    buff.append(domElementId);
    buff.append("\" class=\"");
    buff.append(cssClassNameTreeView + "\" >");
    writeRoot(buff);
    writeUList(buff, root);
    buff.append("</div>\n");
    String selectionNodeId = (selectedIndex==-1) ? "" : String.valueOf(getNodeList().get(selectedIndex).getId());
    buff.append("<input type=\"hidden\" id=\"" + selected() + "\" name=\""
                + selected() + "\" value=\"" + selectionNodeId + "\">\n");
    //buff.append(getSelectionRequest());
    return buff.toString();
  }

  /**
   * 指定されたノードのラベル部分と子要素のHTMLを取得します.<br/>
   * 指定されたノードの<li>・・・</li>のInnerHTMLに相当する部分です。
   * @param node
   * @return ノードのラベル部分と子要素のHTML
   */
  public String getNodeHTML(Node node) {
    StringBuffer buff = new StringBuffer();
    if (node==this.getRoot()) {
      writeRoot(buff);
      writeUList(buff, root);
    } else {
      writeListItem(buff, node);
    }
    return buff.toString();
  }

  void writeRoot(StringBuffer buff) {
    if (isShowRoot()) {
      buff.append("<span class=\"");
      buff.append(getToggle(root.isExpanded()));
      buff.append("\"><span class=\"");
      buff.append(cssClassNameLabel + "\" ");
      buff.append(toLabelId(root));
      buff.append(onClickScript(root));
      buff.append(" >");
      buff.append(root.getName());
      buff.append("</span></span>");
    }
  }

  void writeUList(StringBuffer buff, Node parentNode) {
    if (parentNode.getChildren().size()==0) return;
    buff.append("<ul>");
    for (Node child : parentNode.getChildren()) {
      buff.append("<li ");
      buff.append(toNodeId(child));
      buff.append(">");
      writeListItem(buff, child);
      buff.append("</li>");
    }
    buff.append("</ul>");
  }

  String getItemColor(Node node) {
    if (getNodeList().indexOf(node)==selectedIndex) {
      //return " style=\"color:" + selectedItemColor + "\" ";
    }
    return "";
  }

  void writeListItem(StringBuffer buff, Node node) {
    buff.append("<span class=\"");
    if (node instanceof Leaf) {
      buff.append(cssClassNameLeaf);
      //buff.append("\">");
      if (node.hasLink()) {
        buff.append("\">");//2010-01-29 add
        buff.append("<a href=\"");
        buff.append(node.getLink());
        buff.append("\">");
      } else {
        //2010-01-29 mod
        //buff.append("<span class=\"");
        //buff.append(cssClassNameLabel);
        buff.append("\" ");
        buff.append(getItemColor(node));
        buff.append(toLabelId(node));
        buff.append(onClickScript(node));
        buff.append(">");
      }
      buff.append(HTML.escape(node.getName()));
      if (node.hasLink()) {
        buff.append("</a>");
      } else {
        //2010-01-29 del
        //buff.append("</span>");
      }
    } else {
      buff.append(getToggle(node.isExpanded()));
      buff.append("\">");
      buff.append("<span class=\"");
      buff.append(cssClassNameLabel);
      if (node.hasLink()) {
        buff.append(toLabelId(node));
        buff.append("\">");
        buff.append("<a href=\"");
        buff.append(node.getLink());
        buff.append("\">");
      } else {
        buff.append("\" ");
        buff.append(getItemColor(node));
        buff.append(toLabelId(node));
        buff.append(onClickScript(node));
        buff.append(">");
      }
      buff.append(HTML.escape(node.getName()));
      if (node.hasLink()) {
        buff.append("</a>");
      }
      buff.append("</span>");
    }
    buff.append("</span>");
    writeUList(buff, node);
  }

  private String errorMessage;
  @Override
  public boolean checkRequest(HttpServletRequest request) {
    errorMessage = "";
    try {
      long selectedId;
      String value = request.getParameter(selected());

      if (value!=null && !"".equals(value)) {
        selectedId = Long.parseLong(value);
        Node node = getNodeById(selectedId);
        if (node!=null) {
          selectedIndex = getNodeList().indexOf(node);
        } else {
          selectedIndex = -1;
        }
        return true;
      }
      value = request.getParameter(id + "_expand");
      if (value!=null && !"".equals(value)) {
        selectedId = Long.parseLong(value);
        //System.out.println("@@@ expand node : " + selectedId);
        Node node = getNodeById(selectedId);
        node.expand();
        return true;
      }

      value = request.getParameter(id + "_collapse");
      if (value!=null && !"".equals(value)) {
        selectedId = Long.parseLong(value);
        //System.out.println("@@@ collapse node : " + selectedId);
        Node node = getNodeById(selectedId);
        node.collapse();
        return true;
      }
      return true;
    } catch (Exception x) {
      //errorMessage = "Index values are not correctly included in the request.";
      //return false;
    }
    return true;
  }

//  private Node getNodeById(long selectedId) {
//    for (Node node : getNodeList()) {
//      if (node.getId()==selectedId) {
//        return node;
//      }
//    }
//    return null;
//  }

  @Override
  public String getErrorMessages() {
    return errorMessage;
  }

  @Override
  public String getDefaultInformation() {
    if (selectedIndex==-1) return "";
    return HTML.escape(getPathString(getNodeList().get(selectedIndex)));
  }

  public String getCssClassNameTreeView() {
    return cssClassNameTreeView;
  }

  public void setCssClassNameTreeView(String cssClassNameTreeView) {
    this.cssClassNameTreeView = cssClassNameTreeView;
  }

  public String getCssClassNameNodeExpanded() {
    return cssClassNameNodeExpanded;
  }

  public void setCssClassNameNodeExpanded(String cssClassNameNodeExpanded) {
    this.cssClassNameNodeExpanded = cssClassNameNodeExpanded;
  }

  public String getCssClassNameNodeCollapsed() {
    return cssClassNameNodeCollapsed;
  }

  public void setCssClassNameNodeCollapsed(String cssClassNameNodeCollapsed) {
    this.cssClassNameNodeCollapsed = cssClassNameNodeCollapsed;
  }

  public String getCssClassNameLabel() {
    return cssClassNameLabel;
  }

  public void setCssClassNameLabel(String cssClassNameLabel) {
    this.cssClassNameLabel = cssClassNameLabel;
  }

  public String getCssClassNameLeaf() {
    return cssClassNameLeaf;
  }

  public void setCssClassNameLeaf(String cssClassNameLeaf) {
    this.cssClassNameLeaf = cssClassNameLeaf;
  }

  /**
   * ツリーノードが選択された際、サーバー側で実装されるアクションメソッドの名前を取得します.<br/>
   * ディフォルトは"doDefault"です.
   * @return サーバーアクションメソッド名
   */
  public String getOnSelectionMethodName() {
    return onSelectionMethodName;
  }

  /**
   * ツリーノードが選択された際、サーバー側で実装されるアクションメソッドを名前で指定します.<br/>
   * ディフォルトは\"doDefault\"です.
   * @param サーバーアクションメソッド名
   */
  public void setOnSelectionMethodName(String onSelectionMethodName) {
    this.onSelectionMethodName = onSelectionMethodName;
  }

  /**
   * ツリーノードが選択された際、ブラウザで実行されるJavaScriptのハンドラを名前で指定します.<br/>
   * ディフォルト：nullの場合、"on_" + domElementId + "_SelectionNode" という名前のJavaScriptハンドラが使用されます.
   * @return JavaScriptハンドラ名
   */
  public String getOnSelectionCompleteAjaxHandlerName() {
    return onSelectionCompleteAjaxHandlerName;
  }

  /**
   * ツリーノードが選択された際、ブラウザで実行されるJavaScriptのハンドラを名前で指定します.<br/>
   * ディフォルト：nullの場合、"on_" + domElementId + "_SelectionNode" という名前のハンドラが使用されます.
   * @param JavaScriptハンドラ名
   */
  public void setOnSelectionCompleteAjaxHandlerName(String onSelectionCompleteAjaxHandlerName) {
    this.onSelectionCompleteAjaxHandlerName = onSelectionCompleteAjaxHandlerName;
  }

  @Override
  public String getName() {
    return name;
  }

  @Override
  public void setName(String name) {
    this.name = name;
  }

  @Override
  public String getLoadscript() {
    StringBuffer buff = new StringBuffer();
    //buff.append("<script type=\"text/javascript\"><!--\n");
    //buff.append("function " + onSelectionName() + "(node) {\n");
    buff.append(onSelectionName() + " = function (node, exit) {\n");
    buff.append("  var labelId = 'label' + node;\n");
    buff.append("  try {\n");
    buff.append("    var oldId = 'label' + $('" + selected() + "').value;\n");
    buff.append("    $(oldId).style.color = $(oldId)._oldColor;\n");
    buff.append("  } catch (e) {};\n");
    buff.append("  $('" + selected() + "').value = node;\n");
    buff.append("  $(labelId)._oldColor = $(labelId).style.color;\n");
    buff.append("  $(labelId).style.color = '" + selectedItemColor + "';\n");
    buff.append("  if (exit) return;\n");
    buff.append("  ls.ajax('" + onSelectionMethodName + "',");
    if (onSelectionCompleteAjaxHandlerName != null ) {
      buff.append(" " + onSelectionCompleteAjaxHandlerName + ");");
      buff.append("}\n");
    } else {
      buff.append(" on_" + onSelectionName() + ");");
      buff.append("}\n");
      buff.append("var on_" + onSelectionName() + " = {");
      buff.append(" asynchronous: 'true',");
      buff.append(" onComplete: function(response){");
      buff.append("   Element.Methods.update($('" + defaultInformationName() + "'), response." + name
          + "_defaultInformation);");
      buff.append(" }\n");
      buff.append("}\n");
    }

    String jsTreeOptions = "$('" + getName() + "')._lsUIWidget.options.";

    if (isUseExpandAction()) {
      buff.append(jsTreeOptions + "_onExpand = " + jsTreeOptions + "onExpand;\n");
      buff.append(jsTreeOptions + "onExpand = function(node){\n");
      buff.append("  ls.ajax('" + onExpandName() + "',");
      buff.append(" {asynchronous: 'true', parameters: '" + id + "_expand=' + node.id,");
      buff.append("  onComplete: " + jsTreeOptions + "_onExpand},{nodeId:node.id});\n");
      buff.append("}\n");
    }

    if (isUseCollapseAction()) {
      buff.append(jsTreeOptions + "_onCollapse = " + jsTreeOptions + "onCollapse;\n");
      buff.append(jsTreeOptions + "onCollapse = function(node){\n");
      buff.append("  ls.ajax('" + onCollapseName() + "',");
      buff.append(" {asynchronous: 'true', parameters: '" + id + "_collapse=' + node.id,");
      buff.append("  onComplete: " + jsTreeOptions + "_onCollapse},{nodeId:node.id});\n");
      buff.append("}\n");
    }
    //buff.append("//-->");
    //buff.append("</script>");
    return buff.toString();
  }

  /**
   * 選択された項目の文字色を取得します.
   * @return #FFFFFF形式の色指定
   */
  public String getSelectedItemColor() {
    return selectedItemColor;
  }

  /**
   * 選択された項目の文字色を設定します.
   * @param selectedItemColor #FFFFFF形式の色指定
   */
  public void setSelectedItemColor(String selectedItemColor) {
    this.selectedItemColor = selectedItemColor;
  }

}
